--[[ Core Functions ]]--
function math.round(num, idp)
	if not idp then idp = 0 end
	return tonumber(string.format("%."..idp.."f", num ))
end

function string.upperf(text)
	local up = function(first, rest)
		return string.upper(first)..string.lower(rest)
	end
	text = string.gsub(string.lower(text), "(%l)([%w_']*)", up)
	return text
end

--[[ Clique Support ]]--
ClickCastFrames = ClickCastFrames or {}

nurfed_util = {}
nurfed_util.frametemp = {}
nurfed_util.frameinit = {
	Show = function(obj) obj:Show() end,
	Hide = function(obj) obj:Hide() end,
	size = function(obj, val) obj:SetWidth(val[1]) obj:SetHeight(val[2]) end,
	vars = function(obj, val) for k, v in pairs(val) do obj[k] = v end end,
	events = function(obj, val) for _, v in ipairs(val) do obj:RegisterEvent(v) end end,
	children = function(obj, val)
		if obj:GetName() then
			for k, v in pairs(val) do
				local cobj = nurfed_util:createobj(obj:GetName()..k, v, obj)
				if string.find(k, "Texture") then
					local method = obj["Set"..k]
					if method then
						method(obj, cobj)
					end
				end
			end
		end
	end,
}

nurfed_util.framecomplete = {
	Anchor = {},
	BackdropColor = {},
	BackdropBorderColor = {},
}

function nurfed_util:new()
	local o = {}
	setmetatable(o, self)
	self.__index = self
	return o
end

--[[ Text Handling ]]--
function nurfed_util:print(msg, out, r, g, b)
	if not r then r = 1 end
	if not g then g = 1 end
	if not b then b = 1 end
	if not out then out = 1 end
	out = getglobal("ChatFrame"..out) or DEFAULT_CHAT_FRAME
	out:AddMessage(msg, r, g, b)
end

function nurfed_util:rgbhex(tbl)
	if not tbl then return "|cffffffff" end
	if not tbl.r then tbl.r = tbl[1] end
	if not tbl.g then tbl.g = tbl[2] end
	if not tbl.b then tbl.b = tbl[3] end
	return string.format("|cff%02x%02x%02x", (tbl.r*255), (tbl.g*255), (tbl.b*255))
end

function nurfed_util:binding(bind)
	bind = string.gsub(bind, "CTRL%-", "C-")
	bind = string.gsub(bind, "ALT%-", "A-")
	bind = string.gsub(bind, "SHIFT%-", "S-")
	bind = string.gsub(bind, "Num Pad", "NP")
	bind = string.gsub(bind, "Backspace", "Bksp")
	bind = string.gsub(bind, "Spacebar", "Space")
	bind = string.gsub(bind, "Page", "Pg")
	bind = string.gsub(bind, "Down", "Dn")
	bind = string.gsub(bind, "Arrow", "")
	bind = string.gsub(bind, "Insert", "Ins")
	bind = string.gsub(bind, "Delete", "Del")
	return bind
end

function nurfed_util:formatgs(gstring, anchor)
	gstring = string.gsub(gstring,"([%^%(%)%.%[%]%*%+%-%?])","%%%1")
	gstring = string.gsub(gstring,"%%s","(.+)")
	gstring = string.gsub(gstring,"%%d","(%-?%%d+)")
	if anchor then gstring = "^"..gstring end
	return gstring
end

--[[ Table Handling ]]--
function nurfed_util:copytable(t)
	local new = {}
	local i, v = next(t, nil)
	while i do
		if type(v) == "table" then
			v = self:copytable(v)
		end
		new[i] = v
		i, v = next(t, i)
	end
	return new
end

--[[ Option Handling ]]--
function nurfed_util:getopt(opt)
	if NURFED_SAVED[opt] or type(NURFED_SAVED[opt]) == "boolean" then
		return NURFED_SAVED[opt]
	else
		return NURFED_DEFAULT[opt]
	end
end

--[[ Frame Handling ]]--
function nurfed_util:createtemp(name, layout)
	local objtype = rawget(layout, "type")
	if objtype == "Font" then
		self:createobj(name, layout)
	else
		self.frametemp[name] = layout
	end
end

function nurfed_util:create(name, layout, parent)
	if not parent then parent = UIParent end
	local obj = self:createobj(name, layout, parent)

	for i = #self.framecomplete.Anchor, 1, -1 do
		local tbl = self.framecomplete.Anchor[i]
		local noremove = false
		tbl[1]:ClearAllPoints()
		if (type(tbl[2]) ~= "table") then
			tbl[1]:SetAllPoints(tbl[1]:GetParent())
		else
			local anchor = self:copytable(tbl[2])
			if type(anchor[2]) == "string" then
				anchor[2] = string.gsub(anchor[2], "$parent", tbl[1]:GetParent():GetName())
			end
			if not anchor[2] or type(anchor[2]) == "number" or getglobal(anchor[2]) then
				tbl[1]:SetPoint(unpack(anchor))
			else
				noremove = true
			end
		end
		if not noremove then
			table.remove(self.framecomplete.Anchor, i)
		end
	end

	for i = #self.framecomplete.BackdropColor, 1, -1 do
		local tbl = self.framecomplete.BackdropColor[i]
		tbl[1]:SetBackdropColor(unpack(tbl[2]))
		table.remove(self.framecomplete.BackdropColor, i)
	end

	for i = #self.framecomplete.BackdropBorderColor, 1, -1 do
		local tbl = self.framecomplete.BackdropBorderColor[i]
		tbl[1]:SetBackdropBorderColor(unpack(tbl[2]))
		table.remove(self.framecomplete.BackdropBorderColor, i)
	end

	return obj
end

function nurfed_util:createobj(name, layout, parent)
	if type(parent) == "string" then parent = getglobal(parent) end
	if type(layout) == "string" then layout = self.frametemp[layout] end

	if layout.template then
		local temp = self.frametemp[layout.template] or {}
		for k, v in pairs(temp) do
			if not layout[k] then
				layout[k] = v
			elseif type(layout[k]) == "table" and type(v) == "table" then
				for x, y in pairs(v) do
					if not layout[k][x] then
						layout[k][x] = y
					end
				end
			end
		end
		layout.template = nil
	end
	local objtype = rawget(layout, "type")
	if not objtype then return end
	local obj
	local inherit = layout.uitemp or nil
	if (objtype == "Texture") then
		obj = parent:CreateTexture(name, layout.layer, inherit)
	elseif (objtype == "FontString") then
		obj = parent:CreateFontString(name, layout.layer, inherit)
	elseif objtype == "Font" then
		obj = CreateFont(name)
	else
		obj = CreateFrame(objtype, name, parent, inherit)
	end

	for k, v in pairs(layout) do
		if type(v) == "table" and v.template then
			v = self.frametemp[v.template]
		elseif type(v) == "string" and self.frametemp[v] then
			v = self.frametemp[v]
		end
		if type(v) == "function" and string.find(k, "^On") then
			obj:SetScript(k, v)
		elseif self.frameinit[k] then
			self.frameinit[k](obj, v)
		elseif self.framecomplete[k] then
			table.insert(self.framecomplete[k], { obj, v })
		elseif k == "Point" then
			table.insert(self.framecomplete.Anchor, { obj, v })
		else
			local method = obj["Set"..k] or obj["Enable"..k]
			if method then
				if type(v) == "table" and k ~= "Backdrop" then
					method(obj, unpack(v))
				else
					method(obj, v)
				end
			end
		end
	end
	return obj
end

function nurfed_util:getframes(frame, tbl, recurse)
	if frame.GetChildren then
		local children = { frame:GetChildren() }
		for _, child in pairs(children) do
			if child:GetName() then
				table.insert(tbl, child:GetName())
				if recurse then self:getframes(child, tbl) end
			end
		end
	end

	if frame.GetRegions then
		local regions = { frame:GetRegions() }
		for _, region in pairs(regions) do
			if region:GetName() then
				table.insert(tbl, region:GetName())
			end
		end
	end
end

--[[ Misc Functions ]]--
function nurfed_util:getspells()
	local spells = {}
	local tabs = GetNumSpellTabs()
	for tab = 1, tabs do
		local _, _, offset, numSpells = GetSpellTabInfo(tab)
		spells[tab] = {}
		for i = 1, numSpells do
			local spell = offset + i
			if not IsPassiveSpell(spell, BOOKTYPE_SPELL) then
				local spellname, spellrank = GetSpellName(spell, BOOKTYPE_SPELL)
				if not spells[tab][spellname] then
					spells[tab][spellname] = {}
					table.insert(spells[tab], spellname)
				end
				table.insert(spells[tab][spellname], spell)
			end
		end
	end
	return spells
end

function nurfed_util:getmacros()
	local macros = {}
	macros[1], macros[2] = {}, {}
	local all, char = GetNumMacros()
	for i = 1, all do
		local name, texture = GetMacroInfo(i)
		table.insert(macros[1], { name, texture })
	end
	for i = 19, char + 18 do
		local name, texture = GetMacroInfo(i)
		table.insert(macros[2], { name, texture })
	end
	return macros
end

function nurfed_util:savebind(key, spell, type)
	if not spell then
		SetBinding(key)
		NURFED_BINDINGS[key] = nil
	else
		type = string.upperf(type)
		local func = getglobal("SetBinding"..type)
		func(key, spell)
		NURFED_BINDINGS[key] = { type, spell }
	end
	SaveBindings(GetCurrentBindingSet())
end

function nurfed_util:getunit(name)
	local num = GetNumRaidMembers()
	for i = 1, num do
		local info = { GetRaidRosterInfo(i) }
		if info[1] == name then
			return info
		end
	end
end