--[[ Nurfed Unit Library ]]--
local nrf = nurfed_util:new()

local combatlog = {
	player = {},
	party1 = {},
	party2 = {},
	party3 = {},
	party4 = {},
}

local race = {
	["HUMAN_MALE"]		= {0, 0.125, 0, 0.25},
	["DWARF_MALE"]		= {0.125, 0.25, 0, 0.25},
	["GNOME_MALE"]		= {0.25, 0.375, 0, 0.25},
	["NIGHTELF_MALE"]	= {0.375, 0.5, 0, 0.25},
	["TAUREN_MALE"]		= {0, 0.125, 0.25, 0.5},
	["SCOURGE_MALE"]	= {0.125, 0.25, 0.25, 0.5},
	["TROLL_MALE"]		= {0.25, 0.375, 0.25, 0.5},
	["ORC_MALE"]		= {0.375, 0.5, 0.25, 0.5},
	["HUMAN_FEMALE"]	= {0, 0.125, 0.5, 0.75},
	["DWARF_FEMALE"]	= {0.125, 0.25, 0.5, 0.75},
	["GNOME_FEMALE"]	= {0.25, 0.375, 0.5, 0.75},
	["NIGHTELF_FEMALE"]	= {0.375, 0.5, 0.5, 0.75},
	["TAUREN_FEMALE"]	= {0, 0.125, 0.75, 1.0},
	["SCOURGE_FEMALE"]	= {0.125, 0.25, 0.75, 1.0},
	["TROLL_FEMALE"]	= {0.25, 0.375, 0.75, 1.0},
	["ORC_FEMALE"]		= {0.375, 0.5, 0.75, 1.0},
	["BLOODELF_MALE"]	= {0.5, 0.625, 0.25, 0.5},
	["BLOODELF_FEMALE"]	= {0.5, 0.625, 0.75, 1.0},
	["DRAENEI_MALE"]	= {0.5, 0.625, 0, 0.25},
	["DRAENEI_FEMALE"]	= {0.5, 0.625, 0.5, 0.75},
}

local class = {
	["WARRIOR"]	= {0, 0.25, 0, 0.25},
	["MAGE"]	= {0.25, 0.49609375, 0, 0.25},
	["ROGUE"]	= {0.49609375, 0.7421875, 0, 0.25},
	["DRUID"]	= {0.7421875, 0.98828125, 0, 0.25},
	["HUNTER"]	= {0, 0.25, 0.25, 0.5},
	["SHAMAN"]	= {0.25, 0.49609375, 0.25, 0.5},
	["PRIEST"]	= {0.49609375, 0.7421875, 0.25, 0.5},
	["WARLOCK"]	= {0.7421875, 0.98828125, 0.25, 0.5},
	["PALADIN"]	= {0, 0.25, 0.5, 0.75},
}

local cure = {
	["Magic"] = {
		["PRIEST"] = true,
		["PALADIN"] = true,
		["WARLOCK"] = true,
	},
	["Curse"] = {
		["DRUID"] = true,
		["MAGE"] = true,
	},
	["Disease"] = {
		["PRIEST"] = true,
		["PALADIN"] = true,
		["SHAMAN"] = true,
	},
	["Poison"] = {
		["DRUID"] = true,
		["SHAMAN"] = true,
		["PALADIN"] = true,
	},
}

local damage = {
	{ (255/255), (255/255), (0/255) },
	{ (255/255), (0/255), (0/255) },
	{ (0/255), (102/255), (0/255) },
	{ (0/255), (102/255), (255/255) },
	{ (202/255), (76/255), (217/255) },
	{ (153/255), (204/255), (255/255) },
}

local classification = {
	["rareelite"] = ITEM_QUALITY3_DESC.."-"..ELITE,
	["rare"] = ITEM_QUALITY3_DESC,
	["elite"] = ELITE,
}

local replace = {
	["$faction"] = function(t) return UnitFactionGroup(this.unit) or "" end,
	["$rname"] = function(t) return GetPVPRankInfo(UnitPVPRank(this.unit)) or "" end,
	["$rnum"] = function(t) return select(2, GetPVPRankInfo(UnitPVPRank(this.unit))) or "" end,
	["$race"] = function(t) return UnitRace(this.unit) or "" end,

	["$name"] = function(t)  --fridg
		local name = UnitName(this.unit)
		local color = { r = 1, g = 1, b = 1 }
			local _, eclass = UnitClass(this.unit)
			color = RAID_CLASS_COLORS[eclass]  --fridg removed reaction colors for names
		return nrf:rgbhex(color)..name.."|r"
	end,

	["$realm"] = function(t)  --fridg standard realm name with custom color
		local _, realm = UnitName(this.unit);
		realm = "|cffaf37ff"..realm.."|r";
		return realm or "";
	end,

	["$drealm"] = function(t)  --fridg dashed realm name
		local _, realm = UnitName(this.unit);
		if realm then
			realm = "|cffffffff-|r|cffaf37ff"..realm.."|r";
		end
			return realm or "";
	end,


	["$key"] = function(t)
			local id, found = gsub(this.unit, "party([1-4])", "%1")
			if found == 1 then
				local binding = GetBindingText(GetBindingKey("TARGETPARTYMEMBER"..id), "KEY_")
				binding = nrf:binding(binding)
				return binding or ""
			end
	end,

	["$level"] = function(t)
			local level = UnitLevel(this.unit)
			local classification = UnitClassification(this.unit)
			local color = GetDifficultyColor(level)
			if UnitIsPlusMob(this.unit) then
				level = level.."+"
			elseif level == 0 then
				level = ""
			elseif level < 0 then
				level = "??"
				color = { r = 1, g = 0, b = 0 }
			end
			if classification == "worldboss" then
				level = BOSS
				color = { r = 1, g = 0, b = 0 }
			end
			return nrf:rgbhex(color)..level.."|r"
	end,

	["$level"] = function(t)
			local level = UnitLevel(this.unit)
			local classification = UnitClassification(this.unit)
			local color = GetDifficultyColor(level)
			if UnitIsPlusMob(this.unit) then
				level = level.."+"
			elseif level == 0 then
				level = ""
			elseif level < 0 then
				level = "??"
				color = { r = 1, g = 0, b = 0 }
			end
			if classification == "worldboss" then
				level = BOSS
				color = { r = 1, g = 0, b = 0 }
			end
			return nrf:rgbhex(color)..level.."|r"
	end,

	["$class"] = function(t)
			local class, eclass = UnitClass(this.unit)
			if (UnitIsPlayer(this.unit)) then
				if (RAID_CLASS_COLORS[eclass]) then
					local color = nrf:rgbhex(RAID_CLASS_COLORS[eclass])
					class = color..class.."|r"
				end
			else
				local unitclass = UnitClassification(this.unit)
				if UnitCreatureType(this.unit) == "Beast" and UnitCreatureFamily(this.unit) then --fridg removed friendly humanoids = NPC
					class = UnitCreatureFamily(this.unit)
				else
					class = UnitCreatureType(this.unit)
				end
				if classification[unitclass] then
					class = classification[unitclass].." "..class
				end
			end
			return class or ""
	end,

	["$classs"] = function(t)
			local class, eclass = UnitClass(this.unit)
			if (RAID_CLASS_COLORS[eclass]) then
				local color = nrf:rgbhex(RAID_CLASS_COLORS[eclass])
				class = string.sub(class,1,3)
				class = color..class.."|r"
			end
			return class or ""
	end,

	["$classAZ"] = function(t)
			local class, eclass = UnitClass(this.unit)
			if (RAID_CLASS_COLORS[eclass]) then
				local color = nrf:rgbhex(RAID_CLASS_COLORS[eclass])
				class = string.sub(class,1,1)
				class = color..class.."|r"
			end
			return class or ""
	end,

	["$guild"] = function(t)
			local guild = GetGuildInfo(this.unit)
			if guild then
				local pguild = GetGuildInfo("player")
				local color = { r = 102/255, g = 179/255, b = 255/255 };  --fridg
				if guild == pguild then
					color =  { r = 19/255, g = 240/255, b = 15/255 }; --fridg
				end
				guild = nrf:rgbhex(color)..guild.."|r"
			end
			return guild or ""
	end,
	["$guildAZ"] = function(t)
			local guildAZ = GetGuildInfo(this.unit)
			if guildAZ then
				local pguild = GetGuildInfo("player")
				local color = { r = 0, g = 0.75, b = 1 }
				if guildAZ == pguild then
					color = { r = 1, g = 0, b = 1 }
				end
				guildAZ = string.gsub(guildAZ, "[^A-Z:0-9.]", "")
				guildAZ = nrf:rgbhex(color)..guildAZ.."|r"
			end
			return guildAZ or ""
	end,
}

local disable = {
	player = function() PlayerFrame:UnregisterAllEvents() PlayerFrame:Hide() end,
	target = function() TargetFrame:UnregisterAllEvents() TargetFrame:Hide() ComboFrame:UnregisterAllEvents() ComboFrame:Hide() end,
	party1 = function()
		for i = 1, 4 do
			local party = getglobal("PartyMemberFrame"..i)
			party:UnregisterAllEvents()
			party:Hide()
		end
		ShowPartyFrame = HidePartyFrame
	end,
}

local auratip = function()
	if not this:IsVisible() then return end
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
	local unit = this:GetParent().unit
	if this.isdebuff then
		GameTooltip:SetUnitDebuff(unit, this:GetID(), this.filter)
	else
		GameTooltip:SetUnitBuff(unit, this:GetID(), this.filter)
	end
end

local glide = function(...)
	if this.fade < 1 then
		local e = select(2, ...)
		this.fade = this.fade + e
		if this.fade > 1 then this.fade = 1 end
		local delta = this.endvalue - this.startvalue
		local diff = delta * (this.fade / 1)
		this.startvalue = this.startvalue + diff
		this:SetValue(this.startvalue)
	end
end

--[[ Based on custom fading hits by Tyrone ]]--
local usedBits = {}
local i = 0

local getbit = function()
	local r
	if #usedBits > 0 then
		r = table.remove(usedBits)
	else
		i = i + 1
		r = UIParent:CreateTexture("nrf_fade"..i)
	end
	return r
end

local killbit = function(item)
	table.insert(usedBits, item)
	item:Hide()
	item:SetParent(UIParent)
end

local nrf_fading = function(self, value, flag)
	local lower,upper = self:GetValue(), self.old
	if lower<upper then
		local min,max = self:GetMinMaxValues()
		if self.old > max then self.old = lower return end
		local chunk = getbit()
		chunk:SetTexture(self.texture)
		chunk:SetParent(self)
		local size=self:GetWidth()
		chunk:SetPoint("TOP", self,0,0)
		chunk:SetPoint("BOTTOM", self,0,0)
		chunk:SetPoint("RIGHT",self,(size *-(max-upper)/max),0)
		chunk:SetPoint("LEFT",self, "RIGHT",(size *-(max-lower)/max),0)
		chunk:Show()
		local fadeinfo = {}
		fadeinfo.timeToFade = 1.5
		fadeinfo.mode = "OUT"
		fadeinfo.finishedFunc = killbit
		fadeinfo.finishedArg1 = chunk
		UIFrameFade(chunk, fadeinfo)
	end
	self.old = lower
end

local fade = function(frame)
	local texture = frame:GetStatusBarTexture()
	local name = texture:GetTexture()
	frame.texture = name
	frame.old = 0
	hooksecurefunc(frame, "SetValue", nrf_fading)
end

local addcombat = function()
	local text = date("[%#I:%M:%S]")
	local unit = arg1
	local event = arg2
	local flags = arg3
	local amount = arg4
	local dtype = arg5
	local color = nrf:rgbhex({ 1, 0.647, 0 })

	if event == "HEAL" then
		text = text.." |cff00ff00+"..amount.."|r"
	elseif event == "WOUND" then
		if amount ~= 0 then
			if dtype == 0 then
				color = "|cffff0000"
			else
				color = nrf:rgbhex(damage[dtype])
			end
			text = text.." "..color.."-"..amount.."|r"
		elseif CombatFeedbackText[flags] then
			text = text.." "..color..CombatFeedbackText[flags].."|r"
		else
			text = text.." "..color..CombatFeedbackText["MISS"].."|r"
		end
	elseif event == "ENERGIZE" then
		color = nrf:rgbhex({ 0.41, 0.8, 0.94 })
		text = text.." "..color..amount.."|r"
	elseif CombatFeedbackText[event] then
		text = text.." "..color..CombatFeedbackText[event].."|r"
	end

	if flags == "CRITICAL" then
		text = text.."|cffffff00*|r"
	elseif flags == "CRUSHING" then
		text = text.."|cffffff00^|r"
	elseif flags == "GLANCING" then
		text = text.."|cffffff00-|r"
	end

	table.insert(combatlog[unit], { UnitName(unit), text })
	if #combatlog[unit] > 50 then
		table.remove(combatlog[unit], 1)
	end
end

local updatedamage = function(event, flags, amount, type)
	local text = ""
	local r, g, b = 1, 0.647, 0

	if event == "HEAL" then
		text = "+"..amount
		r, g, b = 0, 1, 0
	elseif event == "WOUND" then
		if amount ~= 0 then
			if type == 0 then
				r, g, b = 1, 0, 0
			else
				local color = damage[type]
				r, g, b = color[1], color[2], color[3]
			end
			text = "-"..amount
		elseif flags == "ABSORB" then
			text = CombatFeedbackText["ABSORB"]
		elseif flags == "BLOCK" then
			text = CombatFeedbackText["BLOCK"]
		elseif flags == "RESIST" then
			text = CombatFeedbackText["RESIST"]
		else
			text = CombatFeedbackText["MISS"]
		end
	elseif event == "IMMUNE" then
		text = CombatFeedbackText[event]
	elseif event == "BLOCK" then
		text = CombatFeedbackText[event]
	elseif event == "ENERGIZE" then
		text = amount
		r, g, b = 0.41, 0.8, 0.94
	else
		text = CombatFeedbackText[event]
	end

	for _, child in ipairs(this.feedback) do
		if child.heal and event == "HEAL" then
			child:AddMessage(text, r, g, b, 1, 1)
		end
		if child.damage and event ~= "HEAL" then
			child:AddMessage(text, r, g, b, 1, 1)
		end
		if not child.damage and not child.heal then
			child:AddMessage(text, r, g, b, 1, 1)
		end
	end
end

local settexture = function(texture, size, fill, value)
	if fill == "top" then
		texture:SetHeight(size)
		texture:SetTexCoord(0, 1, value, 1)
	elseif fill == "bottom" then
		texture:SetHeight(size)
		texture:SetTexCoord(0, 1, 1, value)
	elseif fill == "left" then
		texture:SetWidth(size)
		texture:SetTexCoord(value, 1, 0, 1)
	elseif fill == "right" then
		texture:SetWidth(size)
		texture:SetTexCoord(1, value, 0, 1)
	elseif fill == "vertical" then
		texture:SetHeight(size)
		texture:SetTexCoord(0, 1, 0 + (value / 2), 1 - (value / 2))
	elseif fill == "horizontal" then
		texture:SetWidth(size)
		texture:SetTexCoord(0 + (value / 2), 1 - (value / 2), 0, 1)
	end
end

local gethealth = function(unit)
	local currhp, maxhp = UnitHealth(unit), UnitHealthMax(unit)
	if MobHealth3 and unit == "target" then
		currhp, maxhp = MobHealth3:GetUnitHealth("target", currhp, maxhp)
	end
	if not UnitIsConnected(unit) then
		currhp = 0
	end
	local perc = currhp/maxhp
	local color = {}
	if Nurfed_HealthPercColor then
		color = Nurfed_HealthPercColor(perc, unit) --fridg
	else
		if perc > 0.5 then
			color.r = (1.0 - perc) * 2
			color.g = 1.0
		else
			color.r = 1.0
			color.g = perc * 2
		end
		color.b = 0.0
	end
	perc = format("%.0f", floor(perc * 100))
	return currhp, maxhp, perc, color
end

local getmana = function(unit)
	local currmp, maxmp = UnitMana(unit), UnitManaMax(unit)
	if not UnitIsConnected(unit) then
		currmp = 0
	end
	local perc = format("%.0f", (currmp / maxmp) * 100)
	return currmp, maxmp, perc
end

local getxp = function(unit)
	if not UnitExists(unit) then
		return 0, 0, 0
	end
	local name, reaction, min, max, value = GetWatchedFactionInfo()
	local currxp, maxxp, perc
	local color = { r = 0.58, g = 0.0, b = 0.55 }
	if name then
		currxp = value - min
		maxxp = max - min
		color = FACTION_BAR_COLORS[reaction]
	else
		local exhaustionStateID = GetRestState()
		if (exhaustionStateID == 1) then
			color = { r = 0.0, g = 0.39, b = 0.88 }
		end
		currxp = UnitXP(unit)
		maxxp = UnitXPMax(unit)
	end
	local perc = format("%.0f", (currxp / maxxp) * 100)
	return currxp, maxxp, perc, color
end

--[[ Casting Bar ]]--
local castevent = function()
	local parent = this.parent
	if event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TARGET_CHANGED" or event == "PARTY_MEMBERS_CHANGED" or event == "PLAYER_FOCUS_CHANGED" then
		local nameChannel  = UnitChannelInfo(this.unit)
		local nameSpell  = UnitCastingInfo(this.unit)
		if nameChannel then
			event = "UNIT_SPELLCAST_CHANNEL_START"
			arg1 = this.unit
		elseif nameSpell then
			event = "UNIT_SPELLCAST_START"
			arg1 = this.unit
		else
			this:Hide()
			if parent then parent:Hide() end
			return
		end
	end

	if arg1 ~= this.unit then return end

	local barText = getglobal(this:GetName().."text")
	local barIcon = getglobal(this:GetName().."icon")
	local orient = this:GetOrientation()

	if event == "UNIT_SPELLCAST_START" then
		local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitCastingInfo(this.unit)
		if not name then
			this:Hide()
			if parent then parent:Hide() end
			return
		end

		this:SetStatusBarColor(1.0, 0.7, 0.0)
		this.startTime = startTime / 1000
		this.maxValue = endTime / 1000

		this:SetMinMaxValues(this.startTime, this.maxValue)
		this:SetValue(this.startTime)
		this:SetAlpha(1.0)
		this.holdTime = 0
		this.casting = 1
		this.channeling = nil
		this.fadeOut = nil
		this:Show()
		if barText and barText.format then
			local out = barText.format
			out = string.gsub(out, "$spell", name)
			if orient == "HORIZONTAL" then
				if string.len(nameSubtext) > 3 then --fridg
					nameSubtext = "("..nameSubtext..")"
				end
			end
			out = string.gsub(out, "$rank", nameSubtext)
			if orient == "VERTICAL" then
				local vtext = ""
				out = string.gsub(out, "[^A-Z0-9.]", "") --fridg
				out = string.gsub(out, "R(%d+)", function(s) return " R"..s end)
				for i = 1, string.len(out) do
					vtext = vtext..string.sub(out, i, i).."\n"
				end
				out = vtext
			end
			barText:SetText(out)
		end
		if barIcon then barIcon:SetTexture(texture) end
		if parent then
			parent:Show()
			parent:SetAlpha(1.0)
		end

	elseif event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_CHANNEL_STOP" then
		if not this:IsVisible() then this:Hide() end
		if this:IsShown() then
			this:SetValue(this.maxValue)
			if event == "UNIT_SPELLCAST_STOP" then
				this:SetStatusBarColor(0.0, 1.0, 0.0)
				this.casting = nil
			else
				this.channeling = nil
			end
			this.fadeOut = 1
			this.holdTime = 0
		end
	elseif event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_INTERRUPTED" then
		if this:IsShown() and not this.channeling then
			this:SetValue(this.maxValue)
			this:SetStatusBarColor(1.0, 0.0, 0.0)
			this.casting = nil
			this.channeling = nil
			this.fadeOut = 1
			this.holdTime = GetTime() + CASTING_BAR_HOLD_TIME
			if barText then
--				local text = INTERRUPTED  --fridg (this is locale specific: enUS)
				local text = "INTRP"
				if event == "UNIT_SPELLCAST_FAILED" then
--					text = FAILED  --fridg
					text = "FAIL"
				end
				if orient == "VERTICAL" then
					local vtext = ""
					for i=1, string.len(text) do
						vtext = vtext..string.sub(text, i, i).."\n"
					end
					text = vtext
				end
				barText:SetText(text)
			end
		end
	elseif event == "UNIT_SPELLCAST_DELAYED" then
		if this:IsShown() then
			local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitCastingInfo(this.unit)
			if not name then
				this:Hide()
				if parent then parent:Hide() end
				return
			end
			this.startTime = startTime / 1000
			this.maxValue = endTime / 1000
			this:SetMinMaxValues(this.startTime, this.maxValue)
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_START" then
		local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitChannelInfo(this.unit)
		if not name then
			this:Hide()
			if parent then parent:Hide() end
			return
		end

		this:SetStatusBarColor(0.0, 1.0, 0.0)
		this.startTime = startTime / 1000
		this.endTime = endTime / 1000
		this.duration = this.endTime - this.startTime
		this.maxValue = this.startTime

		this:SetMinMaxValues(this.startTime, this.endTime)
		this:SetValue(this.endTime)
		this:SetAlpha(1.0)
		this.holdTime = 0
		this.casting = nil
		this.channeling = 1
		this.fadeOut = nil
		this:Show()
		if barText and barText.format then
			local out = barText.format
			out = string.gsub(out, "$spell", name)
			if orient == "HORIZONTAL" then
				if string.len(nameSubtext) > 3 then --fridg
					nameSubtext = "("..nameSubtext..")"
				end
			end
			out = string.gsub(out, "$rank", nameSubtext)
			if orient == "VERTICAL" then
				local vtext = ""
				out = string.gsub(out, "[^A-Z0-9.]", "") --fridg
				for i=1, string.len(out) do
					vtext = vtext..string.sub(out, i, i).."\n"
				end
				out = vtext
			end
			barText:SetText(out)
		end
		if barIcon then barIcon:SetTexture(texture) end
		if parent then
			parent:Show()
			parent:SetAlpha(1.0)
		end
	elseif event == "UNIT_SPELLCAST_CHANNEL_UPDATE" then
		if this:IsShown() then
			local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitChannelInfo(this.unit)
			if not name then
				this:Hide()
				if parent then parent:Hide() end
				return
			end
			this.startTime = startTime / 1000
			this.endTime = endTime / 1000
			this.maxValue = this.startTime
			this:SetMinMaxValues(this.startTime, this.endTime)
		end
	end
end

local castupdate = function()
	if this.casting and this:IsShown() then
		local status = GetTime()
		if status > this.maxValue then
			status = this.maxValue
		end
		if status == this.maxValue then
			this:SetValue(this.maxValue)
			this:SetStatusBarColor(0.0, 1.0, 0.0)
			this.casting = nil
			this.flash = 1
			this.fadeOut = 1
			return
		end
		this:SetValue(status)
		local cast = getglobal(this:GetName().."time")
		if cast then
			cast:SetText(string.format("(%.1fs)", this.maxValue - status))
		end
	elseif this.channeling then
		local time = GetTime()
		if time > this.endTime then
			time = this.endTime
		end
		if time == this.endTime then
			this:SetStatusBarColor(0.0, 1.0, 0.0)
			this.channeling = nil
			this.flash = 1
			this.fadeOut = 1
			return
		end
		local barValue = this.startTime + (this.endTime - time)
		this:SetValue(barValue)
		local cast = getglobal(this:GetName().."time")
		if cast then
			cast:SetText(string.format("(%.1fs)", this.endTime - time))
		end
	elseif GetTime() < this.holdTime then
		return
	elseif this.fadeOut then
		local parent = this.parent
		local alpha = this:GetAlpha() - CASTING_BAR_ALPHA_STEP
		if alpha > 0 then
			this:SetAlpha(alpha)
			if parent then parent:SetAlpha(alpha) end
		else
			this.fadeOut = nil
			this:Hide()
			if parent then parent:Hide() end
		end
	end
end

local updateinfo = function(infotype, frame)
	if frame then this = frame end
	if not UnitExists(this.unit) or not this[infotype] then
		return
	end
	local curr, max, perc, color
	if infotype == "hp" then
		curr, max, perc, color = gethealth(this.unit)
	elseif infotype == "mp" then
		curr, max, perc = getmana(this.unit)
	else
		curr, max, perc, color = getxp(this.unit)
	end
	local missing = curr - max
	local maxtext, missingtext = max, missing

	for _, child in ipairs(this[infotype]) do
		local objtype = child:GetObjectType()
		if objtype == "StatusBar" then
			if max == 0 or this["no"..infotype] then
				child:Hide()
			else
				child:SetMinMaxValues(0, max)
				if color then
					child:SetStatusBarColor(color.r, color.g, color.b)
				end
				if child.ani and child.ani == "glide" then
					child.endvalue = curr
					child.fade = 0.4
				else
					child:SetValue(curr)
				end
				child:Show()
			end
		elseif objtype == "FontString" then
			if infotype == "hp" then
				if max >= 1000000 then
					maxtext = format("%.2fm", max/1000000)
				elseif max >= 100000 then
					maxtext = format("%.1fk", max/1000)
				end
				if missing <= -1000000 then
					missingtext = format("%.2fm", missing/1000000)
				elseif missing <= -100000 then
					missingtext = format("%.1fk", missing/1000)
				end
			end
			local text = child.format
			if text then
				if infotype == "hp" then
					if UnitIsDead(this.unit) and not UnitIsGhost(this.unit) then
						text = DEAD
					elseif UnitIsGhost(this.unit) then
						text = "Ghost"
					elseif not UnitIsConnected(this.unit) then
						text = PLAYER_OFFLINE
					end
				end
				text = string.gsub(text, "$cur", curr)
				text = string.gsub(text, "$max", maxtext)
				text = string.gsub(text, "$perc", perc.."%%")
				text = string.gsub(text, "$miss", missingtext)

				if infotype == "xp" then
					local name = GetWatchedFactionInfo()
					if GetXPExhaustion() ~= nil and not name then
						text = string.gsub(text, "$rest", " ("..GetXPExhaustion()..")")
					elseif (name) then
						text = string.gsub(text, "$rest", " ("..name..")")
					else
						text = string.gsub(text, "$rest", "")
					end
				end
				child:SetText(text)
				if child:GetParent() == this and color then
					child:SetTextColor(color.r, color.g, color.b)
				end
			end
		elseif objtype == "Texture" and child.fill then
			if max == 0 or curr == 0 then
				child:Hide()
			else
				local p = curr / max
				local size = child.bar * p
				local p_h1, p_h2

				if child.fill == "top" or child.fill == "bottom" or child.fill == "vertical" then
					p_h1 = child.bar / child.height
				else
					p_h1 = child.bar / child.width
				end

				p_h2 = 1 - p_h1
				settexture(child, size, child.fill, (1-p) * p_h1 + p_h2)
				if color then
					child:SetVertexColor(color.r, color.g, color.b)
				end
				child:Show()
			end
		end
	end
end

local manacolor = function(frame)
	if frame then this = frame end
	if not this.mp then return end
	local color = ManaBarColor[UnitPowerType(this.unit)]
	for _, child in ipairs(this.mp) do
		local objtype = child:GetObjectType()
		if objtype == "StatusBar" then
			child:SetStatusBarColor(color.r, color.g, color.b)
		elseif objtype == "Texture" then
			child:SetVertexColor(color.r, color.g, color.b)
		elseif objtype == "FontString" and child:GetParent() == this then
			child:SetTextColor(color.r, color.g, color.b)
		end
	end
	updateinfo("mp")
end

local updatestatus = function()
	if this.status then
		local icon = this.status
		local objtype = icon:GetObjectType()
		if this.incombat then
			if objtype == "Texture" then
				icon:SetTexCoord(0.5, 1.0, 0, 0.5)
			else
				icon:SetText("Combat")
				icon:SetTextColor(1, 0, 0)
			end
			icon:Show()
		elseif IsResting() then
			if objtype == "Texture" then
				icon:SetTexCoord(0, 0.5, 0, 0.5)
			else
				icon:SetText(TUTORIAL_TITLE30)
				icon:SetTextColor(1, 1, 0)
			end
			icon:Show()
		else
			icon:Hide()
		end
	end
end

local updateraid = function()
	if this.raidtarget then
		local icon = this.raidtarget
		local index = GetRaidTargetIndex(this.unit)
		if index then
			SetRaidTargetIconTexture(icon, index)
			icon:Show()
		else
			icon:Hide()
		end
	end
end

local subtext = function(text)
	if not text then return end
	local pre = string.find(text, "%$%a")
	string.gsub(text, "%$%a+",
		function (s)
			if Nurfed_Replace and Nurfed_Replace[s] then
				text = string.gsub(text, s, Nurfed_Replace[s])
			elseif replace[s] then
				text = string.gsub(text, s, replace[s])
			end
		end
	)
	if pre == 1 then
		local post = string.find(text, "[%a^%|cff]")
		if post and post > pre then
			text = string.gsub(text, "[^%a]", "", 1)
		end
	end
	return text
end

local formattext = function(frame)
	if frame and frame.format then
		local display = subtext(frame.format)
		frame:SetText(display)
	end
end

local updatetext = function()
	if this.text then
		for _, v in ipairs(this.text) do
			formattext(v)
		end
	end
end

local updatehappinness = function()
	local happiness, damagePercentage, loyaltyRate = GetPetHappiness()
	local hasPetUI, isHunterPet = HasPetUI()
	if happiness or isHunterPet then
		local display
		local text = this.name
		local icon = this.happiness
		if text then display = subtext(text.format) end
		if icon then icon:Show() end
		if happiness == 1 then
			if text then text:SetTextColor(1, 0.5, 0) end
			if icon then icon:SetTexCoord(0.375, 0.5625, 0, 0.359375) end
		elseif happiness == 2 then
			if text then text:SetTextColor(1, 1, 0) end
			if icon then icon:SetTexCoord(0.1875, 0.375, 0, 0.359375) end
		elseif happiness == 3 then
			if text then text:SetTextColor(0, 1, 0) end
			if icon then icon:SetTexCoord(0, 0.1875, 0, 0.359375) end
		end
		if text then
			if loyaltyRate < 0 then
				text:SetText("-"..display.."-")
			elseif loyaltyRate > 0 then
				text:SetText("+"..display.."+")
			else
				text:SetText(display)
			end
		end
	end
end

local updatecombo = function()
	if this.combo then
		local comboPoints = GetComboPoints()
		for _, child in ipairs(this.combo) do
			if comboPoints > 0 then
				local objtype = child:GetObjectType()
				if objtype == "FontString" then
					child:SetText(comboPoints)
					if comboPoints == 5 then --fridg
						child:SetTextColor(255/255, 0/255, 0/155)
					elseif comboPoints == 4 then
						child:SetTextColor(255/255, 140/255, 0/155)
					elseif comboPoints == 3 then
						child:SetTextColor(255/255, 255/255, 0/155)
					elseif comboPoints == 2 then
						child:SetTextColor(200/255, 255/255, 75/155)
					elseif comboPoints == 1 then
						child:SetTextColor(0/255, 255/255, 0/155)
					else
						child:SetTextColor(1, 1, 1)
					end
					child:Show()
				elseif objtype == "StatusBar" then
					child:SetMinMaxValues(0, 5)
					child:SetValue(comboPoints)
					child:Show()
				else
					if comboPoints >= child.id then
						child:Show()
					else
						child:Hide()
					end
				end
			else
				child:Hide()
			end
		end
	end
end

local updateleader = function()
	if this.leader then
		local icon = this.leader
		local id, found = gsub(this.unit, "party([1-4])", "%1")
		if this.unit == "player" then
			if IsPartyLeader() then
				icon:Show()
			else
				icon:Hide()
			end
		elseif found == 1 then
			if GetPartyLeaderIndex() == tonumber(id) then
				icon:Show()
			else
				icon:Hide()
			end
		end
	end
end

local updatemaster = function()
	if this.master then
		local icon = this.master
		local id, found = gsub(this.unit, "party([1-4])", "%1")
		local lootMethod, lootMaster = GetLootMethod()
		if this.unit == "player" then
			if lootMaster == 0 and ((GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0)) then
				icon:Show()
			else
				icon:Hide()
			end
		elseif found == 1 then
			if lootMaster == tonumber(id) then
				icon:Show()
			else
				icon:Hide()
			end
		end
	end
end

local updatepvp = function()
	if this.pvp then
		local icon = this.pvp
		local objtype = icon:GetObjectType()
		local factionGroup, factionName = UnitFactionGroup(this.unit)
		if UnitIsPVPFreeForAll(this.unit) then
			if objtype == "Texture" then
				icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA")
			else
				icon:SetText(PVP_ENABLED)
			end
			if this.unit == "player" and not this.pvpenabled then
				this.pvpenabled = true
				PlaySound("igPVPUpdate")
			end
			icon:Show()
		elseif factionGroup and UnitIsPVP(this.unit) then
			if objtype == "Texture" then
				icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..factionGroup)
			else
				icon:SetText(PVP_ENABLED)
			end
			if this.unit == "player" and not this.pvpenabled then
				this.pvpenabled = true
				PlaySound("igPVPUpdate")
			end
			icon:Show()
		else
			if this.unit == "player" and this.pvpenabled then
				this.pvpenabled = nil
			end
			icon:Hide()
		end
	end
end

local aurafade = function(...)
	local e = select(2, ...)
	this.update = this.update + e
	if this.update > 0.04 then
		local now = GetTime()
		local frame, texture, p
		if now - this.flashtime > 0.3 then
			this.flashdct = this.flashdct * (-1)
			this.flashtime = now
		end

		if this.flashdct == 1 then
			p = (1 - (now - this.flashtime + 0.001) / 0.3 * 0.7)
		else
			p = ( (now - this.flashtime + 0.001) / 0.3 * 0.7 + 0.3)
		end
		this:SetAlpha(p)
		this.update = 0
	end
end

local updateauras = function(frame)
	if frame then
		this = frame
		if not UnitExists(this.unit) then
			return
		end
	end
	local total = 0
	local resize = { buff = {}, debuff = {} }
	local button, name, rank, texture, app, dtype, color
	local epsilon = (1e-5) --fridg

	if this.buff then
		for i = 1, #this.buff do
			button = getglobal(this:GetName().."buff"..i)
			name, rank, texture, app = UnitBuff(this.unit, i, this.bfilter)
			if (not (button.oldh)) or (not (button.oldw)) then --fridg
				button.oldh = button:GetHeight()
				button.oldw = button:GetWidth()
			end
			if name then
				getglobal(button:GetName().."Icon"):SetTexture(texture)
				count = getglobal(button:GetName().."Count")
				if app > 1 then
					count:SetText(app)
					count:Show()
				else
					count:Hide()
				end
				button.filter = this.bfilter
					if (button.oldh ~= button:GetHeight()) or (button.oldw ~= button:GetWidth()) then --fridg
						button:SetHeight(button.oldh)
						button:SetWidth(button.oldw)
					end
				button:Show()
				table.insert(resize.buff, button)
			else
				if (button:GetHeight() ~= epsilon) or (button:GetWidth() ~= epsilon) then --fridg
					button:SetHeight(epsilon)
					button:SetWidth(epsilon)
				end
				button:Hide()
			end
		end
	end

	if this.debuff then
		this.cure = nil
		for i = 1, #this.debuff do
			button = getglobal(this:GetName().."debuff"..i)
			local filter = this.dfilter
			if (this.unit == "target" or this.unit == "focus") and not UnitIsFriend("player", this.unit) then
				filter = nil
			end
			name, rank, texture, app, dtype = UnitDebuff(this.unit, i, filter)
			if (not (button.oldh)) or (not (button.oldw)) then	  --fridg
				button.oldh = button:GetHeight()
				button.oldw = button:GetWidth()
			end
			if name then
				getglobal(button:GetName().."Icon"):SetTexture(texture)
				count = getglobal(button:GetName().."Count")
				border = getglobal(button:GetName().."Border")
				if app > 1 then
					count:SetText(app)
					count:Show()
				else
					count:Hide()
				end

				if dtype then
					color = DebuffTypeColor[dtype]
				else
					color = DebuffTypeColor["none"]
				end
				border:SetVertexColor(color.r, color.g, color.b)
				button.filter = this.dfilter
					if (button.oldh ~= button:GetHeight()) or (button.oldw ~= button:GetWidth()) then	 --fridg
						button:SetHeight(button.oldh)
						button:SetWidth(button.oldw)
					end
				button:Show()
				table.insert(resize.debuff, button)

				local class = select(2, UnitClass("player"))
				if UnitIsFriend("player", this.unit) and dtype and cure[dtype] and cure[dtype][class] then
					if not button:GetScript("OnUpdate") then
						button.flashtime = GetTime()
						button.update = 0
						button.flashdct = 1
						button:SetScript("OnUpdate", aurafade)
					end
					this.cure = cure[dtype][class]
				else
					button:SetScript("OnUpdate", nil)
					button:SetAlpha(1)
				end
			else
				if (button:GetHeight() ~= epsilon) or (button:GetWidth() ~= epsilon) then --fridg
					button:SetHeight(epsilon)
					button:SetWidth(epsilon)
				end
				button:Hide()
			end
		end
	end

	if this.aurawidth then
		for _, v in pairs(resize) do
			if #v > 0 then
				local width = v[1]:GetWidth()
				local total = #v * width
				local scale = this.aurawidth / total
				if scale > 1 then scale = 1 end
				for _, j in ipairs(v) do
					j:SetScale(scale)
				end
			end
		end
	end
end

local updaterank = function()
	if this.rank then
		local icon = this.rank
		local rankname, ranknumber = GetPVPRankInfo(UnitPVPRank(this.unit))
		if ranknumber and ranknumber > 0 then
			local objtype = icon:GetObjectType()
			if objtype == "Texture" then
				if ranknumber>9 then
					icon:SetTexture("Interface\\PVPRankBadges\\PVPRank"..ranknumber)
				else
					icon:SetTexture("Interface\\PVPRankBadges\\PVPRank0"..ranknumber)
				end
			else
				icon:SetText(ranknumber)
			end
			icon:Show()
		else
			icon:Hide()
		end
	end
end

local updatename = function()
	if this.name then
		formattext(this.name)
	end
	if this.class then
		local icon = this.class
		local eclass = select(2, UnitClass(this.unit))
		local info = class[eclass]
		if info then
			icon:SetTexCoord(unpack(info))
		end
	end
	if this.race then
		local icon = this.race
		local erace = select(2, UnitRace(this.unit))
		local gender = UnitSex(this.unit)
		if gender == 1 then
			gender = "MALE"
		else
			gender = "FEMALE"
		end
		local info = race[string.upper(erace).."_"..gender]
		if info then
			icon:SetTexCoord(unpack(info))
		end
	end
end

local updatehighlight = function()
	if UnitExists("target") and UnitName("target") == UnitName(this.unit) then
		this:LockHighlight()
	else
		this:UnlockHighlight()
	end
end

local updategroup = function()
	if this.group then
		local text = this.group
		local info = nrf:getunit(UnitName(this.unit))
		if info then
			text:SetText(GROUP..": |cffffff00"..info[3].."|r")
		else
			text:SetText(nil)
		end
	end
end

local updateframe = function()
	if this.hp then updateinfo("hp") end
	if this.mp then manacolor() end
	if this.xp then updateinfo("xp") end
	if this.combo then updatecombo() end
	if this.status then updatestatus() end
	if this.raidtarget then updateraid() end
	if this.leader then updateleader() end
	if this.master then updatemaster() end
	if this.pvp then updatepvp() end
	if this.buff or this.debuff then updateauras() end
	if this.rank then updaterank() end
	if this.text then updatetext() end
	if this.unit == "pet" then updatehappinness() end
	if this.GetHighlightTexture and this:GetHighlightTexture() then
		updatehighlight()
	end
	if this.portrait then
		SetPortraitTexture(this.portrait, this.unit)
	end
	updatename()
end

local totupdate = function(...)
	local e = select(2, ...)
	this.update = this.update + e
	if this.update > 0.25 then
		if UnitExists(this.unit) then
			updateframe()
		end
		this.update = 0
	end
end

local onevent = function()
  if ( this.isParty ) then
  if( HIDE_PARTY_INTERFACE == "1" and GetNumRaidMembers() > 0 ) then
      this:Hide();
  elseif( this:GetAttribute("state-unitexists") ) then
      this:Show();
  else
      this:Hide();
  end
  end
	if not UnitExists(this.unit) then return end

	if event == "PLAYER_ENTERING_WORLD" then
		this.incombat = nil
		updateframe()
	elseif event == "PLAYER_TARGET_CHANGED" then
		if this.unit == "target" then
			updateframe()
		elseif this:GetHighlightTexture() then
			updatehighlight()
		end
	elseif event == "PLAYER_FOCUS_CHANGED" then
		updateframe()
	elseif event == "PLAYER_ENTER_COMBAT" or event == "PLAYER_REGEN_DISABLED" then
		this.incombat = true
		updatestatus()
	elseif event == "PLAYER_LEAVE_COMBAT" or event == "PLAYER_REGEN_ENABLED" then
		this.incombat = nil
		updatestatus()
	elseif event == "PLAYER_UPDATE_RESTING" then
		updatestatus()
	elseif event == "PLAYER_COMBO_POINTS" then
		updatecombo()
	elseif event == "PLAYER_XP_UPDATE" or event == "PLAYER_LEVEL_UP" or event == "UPDATE_FACTION" or event == "UPDATE_EXHAUSTION" or event == "UNIT_PET_EXPERIENCE" then
		updateinfo("xp")
	elseif event == "RAID_TARGET_UPDATE" then
		updateraid()
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if this.isParty then
			updateframe()
		else
			updategroup()
			updateleader()
			updatemaster()
		end
	elseif event == "PARTY_LEADER_CHANGED" then
		updateleader()
		updatemaster()
	elseif event == "PARTY_LOOT_METHOD_CHANGED" then
		updatemaster()
	elseif event == "RAID_ROSTER_UPDATE" then
		updategroup()
	elseif (event == "PLAYER_GUILD_UPDATE") then
		formattext(this.guild)
	elseif (event == "UPDATE_BINDINGS") then
		formattext(this.key)
	elseif (event == "UNIT_PET" and arg1 == this.punit) then
		updateframe()
	elseif (arg1 == this.unit) then
		if (event == "UNIT_HEALTH" or event == "UNIT_MAXHEALTH") then
			updateinfo("hp")
		elseif (event == "UNIT_MANA" or event == "UNIT_ENERGY" or event == "UNIT_RAGE" or event == "UNIT_FOCUS" or event == "UNIT_MAXMANA" or event == "UNIT_MAXENERGY" or event == "UNIT_MAXRAGE" or event == "UNIT_MAXFOCUS") then
			updateinfo("mp")
		elseif (event == "UNIT_COMBAT") then
			if this.feedback then updatedamage(arg2, arg3, arg4, arg5) end
			if combatlog[arg1] then addcombat() end
		elseif (event == "UNIT_AURA") then
			updateauras()
		elseif (event == "UNIT_DISPLAYPOWER") then
			manacolor()
		elseif (event == "UNIT_PORTRAIT_UPDATE") then
			SetPortraitTexture(this.portrait, this.unit)
		elseif (event == "UNIT_FACTION") then
			updatepvp()
		elseif (event == "UNIT_LEVEL") then
			updateinfo("xp")
			formattext(this.level)
		elseif (event == "UNIT_NAME_UPDATE") then
			updatename()
			updatetext()
		elseif (event == "UNIT_DYNAMIC_FLAGS") then
			formattext(this.name)
		elseif (event == "UNIT_CLASSIFICATION_CHANGED") then
			formattext(this.level)
		elseif (event == "UNIT_HAPPINESS") then
			updatehappinness()
		end
	end
end

function nurfed_util:unitimbue(frame)
	local dropdown, menufunc
	local id, found = string.gsub(frame.unit, "party([1-4])", "%1")
	if found == 1 and string.len(frame.unit) > 6 then
		id = nil
		found = nil
	end
	local events = { "PLAYER_ENTERING_WORLD", "UNIT_NAME_UPDATE", }
	local frames = {}
	self:getframes(frame, frames, true)
	if disable[frame.unit] then disable[frame.unit]() end
	frame:RegisterForClicks("AnyUp")
	frame:SetScript("OnEnter", UnitFrame_OnEnter)
	frame:SetScript("OnLeave", UnitFrame_OnLeave)
	if frame:GetParent() == UIParent then
		frame:RegisterForDrag("LeftButton")
		frame:SetMovable(true)
		frame:SetScript("OnDragStart", function() if not NRF_LOCKED then this:StartMoving() end end)
		frame:SetScript("OnDragStop", function() NURFED_FRAMES.frames[this:GetName()].Point = { this:GetPoint() } this:StopMovingOrSizing() end)
	end

	if found == 1 then
		frame.isParty = true
		frame:SetID(tonumber(id))
		table.insert(events, "PARTY_MEMBERS_CHANGED")
		table.insert(events, "RAID_ROSTER_UPDATE")
		table.insert(events, "UNIT_COMBAT")
	elseif frame.unit == "target" then
		table.insert(events, "PLAYER_TARGET_CHANGED")
		table.insert(events, "UNIT_DYNAMIC_FLAGS")
		table.insert(events, "UNIT_CLASSIFICATION_CHANGED")
		frame:SetScript("OnHide", TargetFrame_OnHide)
		frame:SetScript("OnShow", TargetFrame_OnShow)
	elseif frame.unit == "focus" then
		table.insert(events, "PLAYER_FOCUS_CHANGED")
	elseif string.find(frame.unit, "pet", 1, true) then
		if frame.unit == "pet" then
			frame.punit = "player"
		end
		table.insert(events, "UNIT_PET")
	elseif frame.unit == "player" then
		table.insert(events, "UNIT_COMBAT")
	end

	if found == 1 then
		dropdown = getglobal("PartyMemberFrame"..id.."DropDown")
	elseif string.find(frame.unit, "^raid") then
		frame.isRaid = true
		FriendsDropDown.initialize = UnitPopup_ShowMenu(getglobal(UIDROPDOWNMENU_OPEN_MENU), "RAID", this.unit, UnitName(this.unit), this:GetID())
		FriendsDropDown.displayMode = "MENU"
		dropdown = FriendsDropDown
	else
		dropdown = getglobal(string.gsub(frame.unit, "^%l", string.upper).."FrameDropDown")
	end
	if dropdown then
		menufunc = function() ToggleDropDownMenu(1, nil, dropdown, "cursor") end
	end

	SecureUnitButton_OnLoad(frame, frame.unit, menufunc)
--	RegisterUnitWatch(frame)
	if( found == 1 ) then
  RegisterUnitWatch(frame, true)
  else
  RegisterUnitWatch(frame)
  end
	ClickCastFrames[frame] = true

	local name = frame:GetName()
	local regstatus = function(pre, child)
		if not frame[pre] then
			frame[pre] = {}
			if pre == "hp" then
				table.insert(events, "UNIT_HEALTH")
				table.insert(events, "UNIT_MAXHEALTH")
			elseif pre == "mp" then
				table.insert(events, "UNIT_MANA")
				table.insert(events, "UNIT_MAXMANA")
				table.insert(events, "UNIT_ENERGY")
				table.insert(events, "UNIT_MAXENERGY")
				table.insert(events, "UNIT_RAGE")
				table.insert(events, "UNIT_MAXRAGE")
				table.insert(events, "UNIT_FOCUS")
				table.insert(events, "UNIT_MAXFOCUS")
				table.insert(events, "UNIT_DISPLAYPOWER")
			elseif pre == "xp" then
				if frame.unit == "player" then
					table.insert(events, "PLAYER_XP_UPDATE")
					table.insert(events, "PLAYER_LEVEL_UP")
					table.insert(events, "UPDATE_EXHAUSTION")
					table.insert(events, "UPDATE_FACTION")
				elseif frame.unit == "pet" then
					table.insert(events, "UNIT_PET_EXPERIENCE")
				end
			elseif pre == "combo" then
				table.insert(events, "PLAYER_COMBO_POINTS")
			elseif pre == "feedback" then
				table.insert(events, "UNIT_COMBAT")
			elseif pre == "buff" or pre == "debuff" and not string.find(frame.unit, "target", 2, true) then
				table.insert(events, "UNIT_AURA")
			end
		end
		table.insert(frame[pre], child)
	end

	local update = function(child)
		local objtype = child:GetObjectType()
		local childname = string.gsub(child:GetName(), name, "")
		if not string.find(childname, "^target") and not string.find(childname, "^pet") then
			local pre = string.sub(childname, 1, 2)
			if pre == "hp" or pre == "mp" or pre == "xp" then
				regstatus(pre, child)
				if child.ani then
					if child.ani == "glide" then
						child.endvalue = 1
						child.fade = 1
						child.startvalue = 0
						child:SetScript("OnUpdate", glide)
					elseif child.ani == "fade" then
						fade(child)
					end
				end
			elseif string.find(childname, "^combo") then
				regstatus("combo", child)
			elseif string.find(childname, "^pvp") then
				table.insert(events, "UNIT_FACTION")
				frame.pvp = child
			elseif string.find(childname, "^status") then
				table.insert(events, "PLAYER_ENTER_COMBAT")
				table.insert(events, "PLAYER_LEAVE_COMBAT")
				table.insert(events, "PLAYER_REGEN_DISABLED")
				table.insert(events, "PLAYER_REGEN_ENABLED")
				table.insert(events, "PLAYER_UPDATE_RESTING")
				frame.status = child
			elseif objtype == "FontString" then
				if child.format then
					string.gsub(child.format, "%$%a+",
						function (s)
							if s == "$guild" then
								table.insert(events, "PLAYER_GUILD_UPDATE")
								frame.guild = child
							elseif s == "$level" then
								table.insert(events, "UNIT_LEVEL")
								frame.level = child
							elseif s == "$key" then
								table.insert(events, "UPDATE_BINDINGS")
								frame.key = child
							elseif s == "$name" then
								frame.name = child
							end
						end
					)
					regstatus("text", child)
				elseif childname == "group" then
					table.insert(events, "RAID_ROSTER_UPDATE")
					table.insert(events, "PARTY_MEMBERS_CHANGED")
					frame.group = child
				end
			elseif objtype == "Texture" then
				local texture = child:GetTexture()
				if texture then
					if texture == "Interface\\GroupFrame\\UI-Group-LeaderIcon" then
						table.insert(events, "PARTY_LEADER_CHANGED")
						frame.leader = child
					elseif texture == "Interface\\GroupFrame\\UI-Group-MasterLooter" then
						table.insert(events, "PARTY_LOOT_METHOD_CHANGED")
						frame.master = child
					elseif texture == "Interface\\QuestFrame\\UI-QuestTitleHighlight" then
						table.insert(events, "PLAYER_TARGET_CHANGED")
						if (not frame:GetHighlightTexture()) then
							frame:SetHighlightTexture(child)
						end
					elseif texture == "Interface\\TargetingFrame\\UI-RaidTargetingIcons" then
						table.insert(events, "RAID_TARGET_UPDATE")
						frame.raidtarget = child
					elseif texture == "Interface\\PetPaperDollFrame\\UI-PetHappiness" then
						table.insert(events, "UNIT_HAPPINESS")
						frame.happiness = child
					elseif texture == "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes" then
						frame.class = child
					elseif texture == "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Races" then
						frame.race = child
					end
				elseif string.find(childname, "^portrait") or child.isportrait then
					table.insert(events, "UNIT_PORTRAIT_UPDATE")
					frame.portrait = child
				elseif string.find(childname, "^rank") then
					frame.rank = child
				end
			elseif objtype == "PlayerModel" then
				child:RegisterEvent("PLAYER_ENTERING_WORLD")
				child:RegisterEvent("DISPLAY_SIZE_CHANGED")
				child:RegisterEvent("UNIT_MODEL_CHANGED")
				if frame.unit == "target" then
					child:RegisterEvent("PLAYER_TARGET_CHANGED")
				elseif frame.unit == "pet" then
					child:RegisterEvent("UNIT_PET")
				elseif frame.unit == "focus" then
					child:RegisterEvent("PLAYER_FOCUS_CHANGED")
				elseif found == 1 then
					child:RegisterEvent("PARTY_MEMBERS_CHANGED")
				end
				child.unit = frame.unit
				child:SetScript("OnEvent", function() if event == "DISPLAY_SIZE_CHANGED" then this:RefreshUnit() else this:SetUnit(this.unit) end end)
				if not child.full then
					child:SetScript("OnUpdate", function() this:SetCamera(0) end)
				end
			elseif objtype == "MessageFrame" then
				regstatus("feedback", child)
			elseif objtype == "Button" then
				if string.find(childname, "^buff") or string.find(childname, "^debuff") then
					local border = getglobal(child:GetName().."Border")
					local count = getglobal(child:GetName().."Count")
					local icon = getglobal(child:GetName().."Icon")
					if string.find(childname, "^debuff") then
						local id, found = gsub(childname, "debuff([0-9]+)", "%1")
						border:ClearAllPoints()
						border:SetAllPoints(child)
						child:SetID(id)
						child.isdebuff = true
						regstatus("debuff", child)
					else
						local id, found = gsub(childname, "buff([0-9]+)", "%1")
						child:SetID(id)
						border:Hide()
						regstatus("buff", child)
					end
					count:SetFontObject(Nurfed_UnitFontOutline)
					count:ClearAllPoints()
					count:SetPoint("BOTTOMRIGHT", child, "BOTTOMRIGHT", 0, 0)
					count:SetWidth(child:GetWidth())
					count:SetJustifyH("RIGHT")
					count:SetHeight(0)
					icon:ClearAllPoints()
					icon:SetAllPoints(child)
					if frame:GetObjectType() == "Button" then
						child:SetScript("OnEnter", auratip)
					else
						child:SetScript("OnEnter", nil)
						child:SetScript("OnLeave", nil)
					end
				end
			elseif objtype == "StatusBar" then
				if string.find(childname, "^casting") then
					if child:GetParent() ~= frame then
						child.parent = child:GetParent()
					end
					child.unit = frame.unit
					child.casting = nil
					child.channeling = nil
					child.holdTime = 0
					child:RegisterEvent("UNIT_SPELLCAST_START")
					child:RegisterEvent("UNIT_SPELLCAST_STOP")
					child:RegisterEvent("UNIT_SPELLCAST_FAILED")
					child:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
					child:RegisterEvent("UNIT_SPELLCAST_DELAYED")
					child:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
					child:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
					child:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
					child:RegisterEvent("PLAYER_ENTERING_WORLD")
					if frame.unit == "target" then
						child:RegisterEvent("PLAYER_TARGET_CHANGED")
					elseif frame.unit == "focus" then
						child:RegisterEvent("PLAYER_FOCUS_CHANGED")
					elseif found == 1 then
						child:RegisterEvent("PARTY_MEMBERS_CHANGED")
					end
					child:SetScript("OnEvent", castevent)
					child:SetScript("OnUpdate", castupdate)
				end
			end
		elseif objtype == "Button" then
			if string.find(childname, "^target") then
				child.unit = frame.unit..childname
				self:unitimbue(child)
			elseif string.find(childname, "^pet") then
				child.punit = frame.unit
				child.unit = gsub(frame.unit.."pet", "^([^%d]+)([%d]+)[pP][eE][tT]$", "%1pet%2")
				self:unitimbue(child)
			end
		end
	end

	for _, child in ipairs(frames) do
		update(getglobal(child))
	end

	if string.find(frame.unit, "target", 2) then
		frame.update = 0
		frame:SetScript("OnUpdate", totupdate)
	else
		frame:SetScript("OnEvent", onevent)
		for _, v in ipairs(events) do
			frame:RegisterEvent(v)
		end
	end
end

local combat = function()
	local dropdownFrame = getglobal(UIDROPDOWNMENU_INIT_MENU)
	local button = this.value
	local unit = dropdownFrame.unit
	local name = dropdownFrame.name
	local server = dropdownFrame.server
	if button == "NRF_COMBATLOG" then
		local name = UnitName(unit)
		ChatFrame1:AddMessage("-------Combat History "..name.."-------------------")
		for k, v in ipairs(combatlog[unit]) do
			if name == v[1] then
				ChatFrame1:AddMessage(v[2])
			end
		end
		ChatFrame1:AddMessage("-------End--------------------------------------")
	end
end

UnitPopupButtons["NRF_COMBATLOG"] = { text = "Combat History", dist = 0 }
table.insert(UnitPopupMenus["SELF"], "NRF_COMBATLOG")
table.insert(UnitPopupMenus["PARTY"], "NRF_COMBATLOG")
hooksecurefunc("UnitPopup_OnClick", combat)