local nrf = nurfed_util:new()
local bind

Nurfed_OptionsMenus["Bindings"] = {
	template = "nrf_options",
	children = {
		scroll = {
			type = "ScrollFrame",
			size = { 388, 270 },
			Anchor = { "LEFT", "$parent", "LEFT" },
			uitemp = "FauxScrollFrameTemplate",
			OnVerticalScroll = function() FauxScrollFrame_OnVerticalScroll(14, Nurfed_ScrollBindings) end,
			OnShow = function() Nurfed_ScrollBindings() end,
			OnMouseWheel = function() Nurfed_MouseWheelBindings(arg1) end,
		},
	},
	OnReceiveDrag = function() if CursorHasItem() then Nurfed_AddCursorItem() end end,
	OnMouseDown = function() if CursorHasItem() then Nurfed_AddCursorItem() end end,
}

StaticPopupDialogs["NRF_BINDKEY"] = {
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	OnAccept = function()
		local info = Nurfed_OptionsFrameBindings.data
		nrf:savebind(unpack(info))
		Nurfed_OptionsFrameBindings.data = nil
		Nurfed_OptionsFrameBindings.spell = nil
		Nurfed_ScrollBindings()
	end,
	timeout = 10,
	whileDead = 1,
	hideOnEscape = 1,
}

function Nurfed_MouseWheelBindings(arg1)
	if Nurfed_OptionsFrameBindings.spell then
		if arg1 > 0 then
			Nurfed_Binding_OnKeyDown("MOUSEWHEELUP")
		else
			Nurfed_Binding_OnKeyDown("MOUSEWHEELDOWN")
		end
	else
		ScrollFrameTemplate_OnMouseWheel(arg1)
	end
end

function Nurfed_ScrollBindings()
	local spells = nrf:getspells()
	local macros = nrf:getmacros()
	local count = {}
	for i = 1, #spells do
		local name = GetSpellTabInfo(i)
		table.insert(count, { name, "|cff0099ff" })
		for k, v in ipairs(spells[i]) do
			table.insert(count, { v, spells[i][v], "SPELL" })
		end
	end

	table.insert(count, { " ", "|cff00ff00" })

	for i = 1, #macros do
		if i == 1 then
			table.insert(count, { GENERAL_MACROS, "|cff00ff00" })
		else
			local text = string.format(CHARACTER_SPECIFIC_MACROS, UnitName("player"))
			table.insert(count, { text, "|cff00ff00" })
		end
		for k, v in ipairs(macros[i]) do
			table.insert(count, { v[1], v[2], "MACRO" })
		end
	end

	table.insert(count, { " ", "|cffffff00" })
	table.insert(count, { ITEMS, "|cffffff00" })

	for _, v in ipairs(NURFED_ITEMS) do
		table.insert(count, { v, true, "ITEM" })
	end

	local format_row = function(row, num)
		local name = getglobal(row:GetName().."name")
		local binding = getglobal(row:GetName().."binding")
		local icon = getglobal(row:GetName().."icon")
		binding:SetText(nil)
		icon:Hide()
		row.ranks = nil
		row.spell = nil
		if count[num][3] then
			local spell = count[num][1]
			row.spell = spell
			if Nurfed_OptionsFrameBindings.spell == spell then
				row:LockHighlight()
				row:EnableKeyboard(true)
				bind = count[num][3]
			else
				row:UnlockHighlight()
				row:EnableKeyboard(nil)
			end
			if count[num][3] == "SPELL" then
				icon:SetTexture(GetSpellTexture(count[num][2][1], BOOKTYPE_SPELL))
				if #count[num][2] > 1 then
					row.ranks = #count[num][2]
					if Nurfed_OptionsFrameBindings[spell] then
						spell = spell.."("..RANK.." "..Nurfed_OptionsFrameBindings[spell]..")"
					end
				end
			elseif count[num][3] == "MACRO" then
				icon:SetTexture(count[num][2])
			else
				icon:SetTexture(select(10, GetItemInfo(spell)))
			end
			name:SetPoint("LEFT", icon, "RIGHT", 3, 0)
			name:SetText(spell)
			icon:Show()
			binding:SetText(GetBindingKey(count[num][3].." "..spell))
		else
			name:SetText(count[num][2]..count[num][1].."|r")
			name:SetPoint("LEFT", row, "LEFT", 5, 0)
		end
	end

	local frame = Nurfed_OptionsFrameBindingsscroll
	FauxScrollFrame_Update(frame, #count, 19, 14)
	for line = 1, 19 do
		local offset = line + FauxScrollFrame_GetOffset(frame)
		local row = getglobal("Nurfed_BindingsRow"..line)
		if offset <= #count then
			format_row(row, offset)
			row:Show()
		else
			row:Hide()
		end
	end
	CloseDropDownMenus()
end

function Nurfed_Binding_OnKeyDown(arg1)
	local keyPressed = arg1
	local screenshotKey = GetBindingKey("SCREENSHOT")
	if screenshotKey and keyPressed == screenshotKey then
		Screenshot()
		return
	end
	if keyPressed == "ESCAPE" then
		HideUIPanel(Nurfed_OptionsFrame)
		return
	end

	if Nurfed_OptionsFrameBindings.spell then
		if keyPressed == "UNKNOWN" or keyPressed == "SHIFT" or keyPressed == "CTRL" or keyPressed == "ALT" then
			return
		end
		if keyPressed == "MiddleButton" then
			keyPressed = "BUTTON3"
		elseif keyPressed == "Button4" then
			keyPressed = "BUTTON4"
		elseif keyPressed == "Button5" then
			keyPressed = "BUTTON5"
		end
		if IsShiftKeyDown() then keyPressed = "SHIFT-"..keyPressed end
		if IsControlKeyDown() then keyPressed = "CTRL-"..keyPressed end
		if IsAltKeyDown() then keyPressed = "ALT-"..keyPressed end
		local spell = Nurfed_OptionsFrameBindings.spell
		if Nurfed_OptionsFrameBindings[spell] then
			spell = spell.."("..RANK.." "..Nurfed_OptionsFrameBindings[spell]..")"
		end
		local key = GetBindingKey(bind.." "..spell)
		local text, data
		if key == keyPressed then
			text = "Unbind |cff00ff00"..spell.."|r?"
			Nurfed_OptionsFrameBindings.data = { keyPressed }
		else
			text = "Bind |cff00ff00"..spell.."|r to |cffff0000"..keyPressed.."|r?"
			local oldkey = GetBindingAction(keyPressed)
			if getglobal("BINDING_NAME_"..oldkey) then
				oldkey = getglobal("BINDING_NAME_"..oldkey)
			end
			if oldkey ~= "" then
				text = "Current: |cff00ff00"..oldkey.."|r\n"..text
			end
			Nurfed_OptionsFrameBindings.data = { keyPressed, spell, bind }
		end
		StaticPopupDialogs["NRF_BINDKEY"].text = text
		StaticPopup_Show("NRF_BINDKEY")
	end
end

function Nurfed_Binding_OnClick(button)
	if not this.spell then return end
	if button == "LeftButton" then
		local spellname = this.spell
		if Nurfed_OptionsFrameBindings.spell == spellname then
			Nurfed_OptionsFrameBindings.spell = nil
		else
			Nurfed_OptionsFrameBindings.spell = spellname
		end
		if Nurfed_OptionsFrameBindings[spellname] then
			spellname = spellname.." ("..RANK.." "..Nurfed_OptionsFrameBindings[spellname]..")"
		end
		Nurfed_ScrollBindings()
	elseif button == "RightButton" then
		if this.ranks then
			local dropdown = getglobal(this:GetName().."dropdown")
			local info = {}
			local spell = this.spell
			dropdown.displayMode = "MENU"
			dropdown.initialize = function ()
				info.text = spell
				info.isTitle = 1
				info.notCheckable = 1
				UIDropDownMenu_AddButton(info)

				info = {}
				info.text = "Max "..RANK
				info.func = function() Nurfed_Binding_Dropdown(spell) end
				info.isTitle = nil
				info.notCheckable = 1
				UIDropDownMenu_AddButton(info)

				for i = 1, this.ranks do
					info = {}
					info.text = "("..RANK.." "..i..")"
					info.func = function() Nurfed_Binding_Dropdown(spell, i) end
					info.isTitle = nil
					info.notCheckable = 1
					UIDropDownMenu_AddButton(info)
				end
			end
			ToggleDropDownMenu(1, nil, dropdown, "cursor")
		end
	else
		Nurfed_Binding_OnKeyDown(button)
	end
end

function Nurfed_Binding_Dropdown(spell, rank)
	if rank then
		Nurfed_OptionsFrameBindings[spell] = rank
	elseif Nurfed_OptionsFrameBindings[spell] then
		Nurfed_OptionsFrameBindings[spell] = nil
	end
	Nurfed_ScrollBindings()
end