--[[ Globals ]]--
Nurfed_OptionsMenus = {}
NURFED_ITEMS = NURFED_ITEMS or {}

--[[ Locals ]]--
local nrf = nurfed_util:new()
local menus = {}
local activemenu
local cursoritem

--[[ Static Popups ]]--
StaticPopupDialogs["NRF_RELOADUI"] = {
	text = "Reload User Interface?",
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	OnAccept = function()
		ReloadUI()
	end,
	timeout = 10,
	whileDead = 1,
	hideOnEscape = 1,
}

local saveopt = function()
	local value
	local objtype = this:GetObjectType()
	local func = this.func
	if objtype == "CheckButton" then
		value = this:GetChecked() or false
		local sound = "igMainMenuOptionCheckBoxOff"
		if this:GetChecked() then sound = "igMainMenuOptionCheckBoxOn" end
		PlaySound(sound)
	elseif objtype == "Slider" then
		value = this:GetValue()
	elseif objtype == "EditBox" and this.focus then
		value = this:GetText()
	end

	NURFED_SAVED[this.option] = value
	if func then func() end
end

local onshow = function()
	local text = getglobal(this:GetName().."Text")
	local value = getglobal(this:GetName().."value")
	text:SetText(this.text)
	if this.right then
		if value then
			value:ClearAllPoints()
			value:SetPoint("RIGHT", this, "LEFT", -3, 0)
		else
			text:ClearAllPoints()
			text:SetPoint("RIGHT", this:GetName(), "LEFT", -1, 1)
		end
	end
	if this.color then text:SetTextColor(unpack(this.color)) end
	if this.option then
		local objtype = this:GetObjectType()
		local option = nrf:getopt(this.option)
		if objtype == "CheckButton" then
			this:SetChecked(option)
		elseif objtype == "Slider" then
			local low = getglobal(this:GetName().."Low")
			local high = getglobal(this:GetName().."High")
			low:SetText(this.low)
			high:SetText(this.high)
			this:SetMinMaxValues(this.min, this.max)
			this:SetValueStep(this.step)
			this:SetValue(option)
			value.option = this.option
			value.func = this.func
			Nurfed_Options_sliderOnValueChanged()
		elseif objtype == "EditBox" then
			this:SetText(option)
		end
	end
	this:SetScript("OnShow", nil)
end

local templates = {
	nrf_menu_button = {
		type = "Button",
		size = { 80, 14 },
		TextFontObject = "GameFontNormalSmall",
		TextColor = { 0.5, 0.5, 0.5 },
		HighlightTextColor = { 1, 1, 1 },
		DisabledTextColor = { 1, 1, 1 },
		PushedTextOffset = { 1, -1 },
		children = {
			NormalTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Anchor = "all",
				Texture = NRF_IMG.."statusbar8",
				Gradient = { "VERTICAL", 1, 0.5, 0, 0.2, 0, 0 },
			},
			DisabledTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Anchor = "all",
				Texture = NRF_IMG.."statusbar8",
				Gradient = { "VERTICAL", 0, 0.75, 1, 0, 0, 0.2 },
			},
		},
	},
	nrf_options = {
		type = "Frame",
		size = { 411, 271 },
		Anchor = { "TOPRIGHT", "$parentheader", "BOTTOMRIGHT", 1, 0 },
		Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 8, insets = { left = 2, right = 2, top = 2, bottom = 2 }, },
		BackdropColor = { 0, 0, 0, 0.95 },
		Alpha = 0,
		Hide = true,
	},
	nrf_addons_row = {
		type = "Frame",
		size = { 400, 14 },
		children = {
			check = {
				type = "CheckButton",
				size = { 16, 16 },
				uitemp = "UICheckButtonTemplate",
				Anchor = { "BOTTOMLEFT", "$parent", "BOTTOMLEFT", 2, 0 },
				OnClick = function() Nurfed_ToggleAddOn() end,
			},
			name = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 190, 14 },
				Anchor = { "LEFT", "$parentcheck", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 1, 1 },
			},
			loaded = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 105, 14 },
				Anchor = { "LEFT", "$parentname", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 1, 1 },
			},
			reload = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 100, 14 },
				Anchor = { "LEFT", "$parentloaded", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 0, 0 },
			},
		},
	},
	nrf_bindings_row = {
		type = "Button",
		size = { 400, 14 },
		children = {
			dropdown = { type = "Frame" },
			name = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 190, 14 },
				Anchor = { "LEFT", "$parent", "LEFT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 1, 1 },
			},
			icon = {
				type = "Texture",
				layer = "ARTWORK",
				size = { 14, 14 },
				Anchor = { "LEFT", "$parent", "LEFT", 15, 0 },
			},
			binding = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 100, 14 },
				Anchor = { "LEFT", "$parentname", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 0, 0 },
			},
			HighlightTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Texture = "Interface\\QuestFrame\\UI-QuestTitleHighlight",
				BlendMode = "ADD",
				Anchor = "all",
			},
		},
		OnClick = function() if CursorHasItem() then Nurfed_AddCursorItem() else Nurfed_Binding_OnClick(arg1) end end,
		OnKeyDown = function() Nurfed_Binding_OnKeyDown(arg1) end,
		OnReceiveDrag = function() if CursorHasItem() then Nurfed_AddCursorItem() end end,
	},
	nrf_templates_row = {
		type = "Button",
		size = { 400, 14 },
		children = {
			icon = {
				type = "Texture",
				Texture = "Interface\\Buttons\\UI-PlusButton-Up",
				layer = "ARTWORK",
				size = { 14, 14 },
				Anchor = { "LEFT", "$parent", "LEFT", 5, 0 },
			},
			name = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 250, 14 },
				Anchor = { "LEFT", "$parenticon", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 1, 1 },
			},
			HighlightTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Texture = "Interface\\QuestFrame\\UI-QuestTitleHighlight",
				BlendMode = "ADD",
				Anchor = "all",
			},
		},
		OnClick = function() Nurfed_Template_OnClick(arg1) end,
	},
	nrf_frames_row = {
		type = "Button",
		size = { 400, 14 },
		children = {
			icon = {
				type = "Texture",
				Texture = "Interface\\Buttons\\UI-PlusButton-Up",
				layer = "ARTWORK",
				size = { 14, 14 },
				Anchor = { "LEFT", "$parent", "LEFT", 5, 0 },
			},
			name = {
				type = "FontString",
				layer = "ARTWORK",
				size = { 250, 14 },
				Anchor = { "LEFT", "$parenticon", "RIGHT", 5, 0 },
				FontObject = "GameFontNormal",
				JustifyH = "LEFT",
				TextColor = { 1, 1, 1 },
			},
			HighlightTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Texture = "Interface\\QuestFrame\\UI-QuestTitleHighlight",
				BlendMode = "ADD",
				Anchor = "all",
			},
		},
		OnClick = function() Nurfed_Frame_OnClick(arg1) end,
	},
	nrf_button = {
		type = "Button",
		size = { 30, 18 },
		Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 10, insets = { left = 2, right = 2, top = 2, bottom = 2 }, },
		BackdropColor = { 0, 0, 0, 0.75 },
		Font = { "Fonts\\2002.TTF", 10, "OUTLINE" },
		TextColor = { 0.65, 0.65, 0.65 },
		HighlightTextColor = { 1, 1, 1 },
		PushedTextOffset = { 1, -1 },
		OnShow = function() this:SetWidth(this:GetTextWidth() + 10) this:SetScript("OnShow", nil) end,
	},
	nrf_check = {
		type = "CheckButton",
		size = { 20, 20 },
		uitemp = "UICheckButtonTemplate",
		OnShow = function() onshow() end,
		OnClick = function() saveopt() end,
	},
	nrf_radio = {
		type = "CheckButton",
		size = { 14, 14 },
		uitemp = "UIRadioButtonTemplate",
		OnShow = function() onshow() end,
		OnClick = function() Nurfed_Options_radioOnClick() end,
	},
	nrf_slider = {
		type = "Slider",
		uitemp = "OptionsSliderTemplate",
		children = {
			value = {
				template = "nrf_editbox",
				size = { 35, 18 },
				Anchor = { "LEFT", "$parent", "RIGHT", 3, 0 },
				OnTextChanged = function()
					local value = tonumber(this:GetText())
					local min, max = this:GetParent():GetMinMaxValues()
					if not value or value < min then return end
					if value > max then value = max end
					this:GetParent():SetValue(value)
					saveopt()
				end,
				OnEditFocusGained = function() this:HighlightText() this.focus = true end,
				OnEditFocusLost = function() this:HighlightText(0, 0) this.focus = nil end,
			},
		},
		OnShow = function() onshow() end,
		OnMouseUp = function() saveopt() end,
		OnValueChanged = function() Nurfed_Options_sliderOnValueChanged() end,
	},
	nrf_editbox = {
		type = "EditBox",
		AutoFocus = false,
		size = { 135, 18 },
		Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 10, insets = { left = 2, right = 2, top = 2, bottom = 2 }, },
		BackdropColor = { 0, 0, 0.2, 0.75 },
		FontObject = "GameFontNormalSmall",
		TextColor = { 1, 1, 1 },
		TextInsets = { 3, 3, 3, 3 },
		children = {
			Text = {
				type = "FontString",
				layer = "ARTWORK",
				Anchor = { "BOTTOMLEFT", "$parent", "TOPLEFT", 3, 0 },
				FontObject = "GameFontNormalSmall",
				JustifyH = "LEFT",
			},
		},
		OnShow = function() onshow() end,
		OnEscapePressed = function() this:ClearFocus() end,
		OnEditFocusGained = function() this:HighlightText() this.focus = true end,
		OnEditFocusLost = function() this:HighlightText(0, 0) this.focus = nil end,
		OnTextChanged = function() saveopt() end,
	},
	nrf_dropdown = {
		type = "Frame",
		uitemp = "UIDropDownMenuTemplate",
		OnShow = function() Nurfed_Options_dropdownOnShow() end,
	},
	nrf_color = {
		type = "Button",
		size = { 18, 18 },
		children = {
			bg = {
				type = "Texture",
				Texture = "Interface\\ChatFrame\\ChatFrameColorSwatch",
				layer = "BACKGROUND",
				Anchor = "all",
				VertexColor = { 1, 1, 1 },
			},
			Text = {
				type = "FontString",
				layer = "ARTWORK",
				Anchor = { "LEFT", "$parent", "RIGHT", 1, 0 },
				FontObject = "GameFontNormalSmall",
				JustifyH = "LEFT",
			},
		},
		OnShow = function() Nurfed_Options_swatchOnShow() end,
		OnClick = function() Nurfed_Options_swatchOpenColorPicker() end,
	},
	nrf_scroll = {
		type = "ScrollFrame",
		Anchor = "all",
		uitemp = "FauxScrollFrameTemplate",
		OnVerticalScroll = function() FauxScrollFrame_OnVerticalScroll(100, Nurfed_Options_ScrollMenu) end,
		OnShow = function() Nurfed_Options_ScrollMenu() end,
	},
	nrf_optionpane = {
		type = "Frame",
		Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 8, insets = { left = 2, right = 2, top = 2, bottom = 2 }, },
		BackdropColor = { 0, 0, 0, 0.5 },
		BackdropBorderColor = { 0.75, 0.75, 0.75, 1 },
		children = {
			title = {
				type = "FontString",
				layer = "ARTWORK",
				Anchor = { "BOTTOMLEFT", "$parent", "TOPLEFT", 10, -2 },
				FontObject = "GameFontNormalSmall",
				JustifyH = "LEFT",
			},
		},
		OnShow = function() Nurfed_Options_paneOnShow() end,
	},
	nrf_paneeditbox = {
		type = "EditBox",
		AutoFocus = false,
		Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = nil, tile = true, tileSize = 16, edgeSize = 8, insets = { left = 2, right = 2, top = 2, bottom = 2 }, },
		BackdropColor = { 0.5, 0.5, 0.5, 0.85 },
		FontObject = "GameFontNormalSmall",
		TextColor = { 1, 1, 0 },
		TextInsets = { 3, 3, 3, 3 },
		OnEscapePressed = function() this:ClearFocus() end,
		OnEditFocusLost = function() this:HighlightText(0, 0) end,
		OnEditFocusGained = function() this:HighlightText() end,
		OnEnterPressed = function() Nurfed_Options_paneAddOption() end,
	},
	nrf_panescroll = {
		type = "ScrollFrame",
		Anchor = "all",
		uitemp = "FauxScrollFrameTemplate",
		Scale = 0.75,
		OnVerticalScroll = function() FauxScrollFrame_OnVerticalScroll(13, Nurfed_Options_ScrollPane) end,
		OnShow = function() Nurfed_Options_ScrollPane() end,
	},
	nrf_pane_row = {
		type = "Button",
		children = {
			text = {
				type = "FontString",
				layer = "ARTWORK",
				Anchor = "all",
				FontObject = "GameFontNormalSmall",
				JustifyH = "LEFT",
				TextColor = { 0, 1, 1 },
			},
			HighlightTexture = {
				type = "Texture",
				layer = "BACKGROUND",
				Texture = "Interface\\QuestFrame\\UI-QuestTitleHighlight",
				BlendMode = "ADD",
				Anchor = "all",
			},
		},
		OnClick = function() Nurfed_Options_PaneSelect() end,
	},
}

local layout = {
	type = "Frame",
	size = { 500, 300 },
	FrameStrata = "LOW",
	Toplevel = true,
	Movable = true,
	Mouse = true,
	ClampedToScreen = true,
	Anchor = { "CENTER", nil, "CENTER", 0, 0 },
	Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 4, top = 5, bottom = 4 }, },
	BackdropColor = { 0, 0, 0, 0.25 },
	children = {
		header = {
			type = "Frame",
			size = { 490, 20 },
			Anchor = { "TOP", "$parent", "TOP", 0, -5 },
			children = {
				bg = {
					type = "Texture",
					layer = "BACKGROUND",
					Anchor = "all",
					Texture = NRF_IMG.."statusbar8",
					Gradient = { "HORIZONTAL", 0, 0.75, 1, 0, 0, 0.2 },
				},
				title = {
					type = "FontString",
					layer = "ARTWORK",
					Anchor = "all",
					Font = { "Fonts\\2002.TTF", 13, "OUTLINE" },
					JustifyH = "LEFT",
					TextColor = { 1, 1, 1 },
				},
				version = {
					type = "FontString",
					layer = "ARTWORK",
					Anchor = "all",
					Font = { "Fonts\\2002.ttf", 13, "NONE" },
					JustifyH = "RIGHT",
					TextColor = { 1, 1, 1 },
				},
				border = {
					type = "Texture",
					size = { 490, 3 },
					layer = "OVERLAY",
					Anchor = { "TOP", "$parent", "BOTTOM", 0, 1 },
					Texture = "Interface\\ClassTrainerFrame\\UI-ClassTrainer-HorizontalBar",
					TexCoord = { 0.2, 1, 0, 0.25 },
				},
			},
		},
		menubg = {
			type = "Frame",
			FrameStrata = "BACKGROUND",
			size = { 85, 278 },
			Anchor = { "TOPLEFT", "$parentheader", "BOTTOMLEFT", -3, 5 },
			Backdrop = { bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 8, insets = { left = 3, right = 2, top = 3, bottom = 2 }, },
			BackdropColor = { 0, 0, 0, 0.95 },
		},
		button1 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentheader", "BOTTOMLEFT", 0, 0 },
			Text = "AddOns",
			ID = 1,
			OnClick = function() Nurfed_MenuClick() end,
		},
		button2 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton1", "BOTTOMLEFT", 0, -1 },
			Text = "Profiles",
			ID = 2,
			OnClick = function() Nurfed_MenuClick() end,
		},
		button3 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton2", "BOTTOMLEFT", 0, -1 },
			Text = "Bindings",
			ID = 3,
			OnClick = function() Nurfed_MenuClick() end,
		},
		button4 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton3", "BOTTOMLEFT", 0, -1 },
			Text = "Templates",
			ID = 4,
			OnClick = function() Nurfed_MenuClick() end,
		},
		button5 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton4", "BOTTOMLEFT", 0, -1 },
			Text = "Frames",
			ID = 5,
			OnClick = function() Nurfed_MenuClick() end,
		},
		button6 = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton5", "BOTTOMLEFT", 0, -1 },
			Text = "General",
			ID = 6,
			OnClick = function() Nurfed_MenuClick() end,
		},
		reloadui = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentbutton6", "BOTTOMLEFT", 0, -1 },
			OnClick = function() StaticPopup_Show("NRF_RELOADUI") end,
			Text = "Reload UI",
		},
		close = {
			template = "nrf_menu_button",
			Anchor = { "TOPLEFT", "$parentreloadui", "BOTTOMLEFT", 0, -1 },
			OnClick = function() HideUIPanel(this:GetParent()) end,
			Text = CLOSE,
		},
	},
	OnDragStart = function() this:StartMoving() end,
	OnDragStop = function() this:StopMovingOrSizing() end,
	Hide = true,
}

function Nurfed_MenuClick()
	this:Disable()
	local id = this:GetID()
	local menu = this:GetText()
	local name = string.gsub(this:GetName(), id, "")
	for i = 1, 6 do
		local button = getglobal(name..i)
		if i ~= id and button:IsEnabled() == 0 then
			button:Enable()
			Nurfed_HideMenu(button:GetText())
		end

		if id == 4 or id == 5 then
			Nurfed_OptionsFrameEditor:Show()
			UIFrameFadeIn(Nurfed_OptionsFrameEditor, 0.25)
		else
			Nurfed_OptionsFrameEditor:SetAlpha(0)
			Nurfed_OptionsFrameEditor:Hide()
		end
	end
	Nurfed_ShowMenu(menu)
end

function Nurfed_ShowMenu(menu)
	if not menus[menu] then
		local opt = nrf:create("Nurfed_OptionsFrame"..menu, Nurfed_OptionsMenus[menu], Nurfed_OptionsFrame)
		menus[menu] = opt
		Nurfed_OptionsMenus[menu] = nil
		if menu == "AddOns" or menu == "Bindings" or menu == "Frames" or menu == "Templates" then
			Nurfed_GenerateMenu(menu)
		end
	end

	PlaySound("igAbiliityPageTurn")
	activemenu = menu
	menus[menu]:Show()
	UIFrameFadeIn(menus[menu], 0.25)
end

function Nurfed_HideMenu(menu)
	menus[menu]:SetAlpha(0)
	menus[menu]:Hide()
end

function Nurfed_GenerateMenu(menu)
	local parent = getglobal("Nurfed_OptionsFrame"..menu)
	local temp = "nrf_"..string.lower(menu).."_row"
	for i = 1, 19 do
		local row = nrf:create("Nurfed_"..menu.."Row"..i, temp, parent)
		if (row:GetObjectType() == "Button") then
			row:RegisterForClicks("AnyUp")
		end
		if (i == 1) then
			row:SetPoint("TOPLEFT", "Nurfed_OptionsFrame"..menu, "TOPLEFT", 0, -3)
		else
			row:SetPoint("TOPLEFT", "Nurfed_"..menu.."Row"..i - 1, "BOTTOMLEFT", 0, 0)
		end
	end
	
end

function Nurfed_AddCursorItem()
	if IsConsumableItem(cursoritem) or GetItemSpell(cursoritem) then
		for _, v in ipairs(NURFED_ITEMS) do
			if v == cursoritem then return end
		end
		table.insert(NURFED_ITEMS, cursoritem)
		table.sort(NURFED_ITEMS, function(a, b) return a > b end)
		Nurfed_ScrollBindings()
	end
	ClearCursor()
end

-----------------------------------------------------------------------------------------
--			Nurfed Options Functions
-----------------------------------------------------------------------------------------

-- scroll
function Nurfed_Options_ScrollMenu()
	FauxScrollFrame_Update(this, this.pages, 1, 100)
	local page = FauxScrollFrame_GetOffset(this) + 1
	local children = { this:GetParent():GetChildren() }
	for _, child in ipairs(children) do
		if (not string.find(child:GetName(), "scroll", 1, true)) then
			if (child.page == page) then
				child:Show()
			else
				child:Hide()
			end
		end
	end
end

-- sliders
function Nurfed_Options_sliderOnValueChanged()
	local value = math.round(this:GetValue(), this.deci)
	local text = getglobal(this:GetName().."value")
	text:SetText(value)
end

-- color swatches
function Nurfed_Options_swatchSetColor(frame)
	local option = frame.option
	local r,g,b = ColorPickerFrame:GetColorRGB()
	local a = OpacitySliderFrame:GetValue()
	local swatch = getglobal(frame:GetName().."bg")
	swatch:SetVertexColor(r, g, b)
	frame.r = r
	frame.g = g
	frame.b = b
	NURFED_SAVED[frame.option] = { r, g, b, a }
	--nrf:SetOption(activemenu, frame.option, { r, g, b, a })
	if (frame.func) then
		frame.func()
	end
end

function Nurfed_Options_swatchCancelColor(frame, prev)
	local option = frame.option
	local r = prev.r
	local g = prev.g
	local b = prev.b
	local a = prev.a
	local swatch = getglobal(frame:GetName().."bg")
	swatch:SetVertexColor(r, g, b)
	frame.r = r
	frame.g = g
	frame.b = b
	NURFED_SAVED[frame.option] = { r, g, b, a }
	--nrf:SetOption(activemenu, frame.option, { r, g, b, a })
	if (frame.func) then
		frame.func()
	end
end

function Nurfed_Options_swatchOnShow()
	if (this:IsShown()) then
		local option = optionInit()
		if (not option) then
			return
		end
		local frame = this
		local swatch = getglobal(this:GetName().."bg")
		swatch:SetVertexColor(option[1], option[2], option[3])

		this.r = option[1]
		this.g = option[2]
		this.b = option[3]
		this.swatchFunc = function() Nurfed_Options_swatchSetColor(frame) end
		this.cancelFunc = function(x) Nurfed_Options_swatchCancelColor(frame, x) end
		if (frame.opacity) then
			this.hasOpacity = frame.opacity
			this.opacityFunc = function() Nurfed_Options_swatchSetColor(frame) end
			this.opacity = option[4]
		end
	end
end

function Nurfed_Options_swatchOpenColorPicker()
	CloseMenus()
	ColorPickerFrame.func = this.swatchFunc
	ColorPickerFrame.hasOpacity = this.hasOpacity
	ColorPickerFrame.opacityFunc = this.opacityFunc
	ColorPickerFrame.opacity = this.opacity
	ColorPickerFrame:SetColorRGB(this.r, this.g, this.b)
	ColorPickerFrame.previousValues = {r = this.r, g = this.g, b = this.b, a = this.opacity}
	ColorPickerFrame.cancelFunc = this.cancelFunc
	ColorPickerFrame:Show()
end

-- radios
function Nurfed_Options_radioOnClick(frame, index, noupdate)
	if (not index) then
		index = this.index
	end
	if (not frame) then
		frame = this:GetParent()
	end
	local children = { frame:GetChildren() }
	for _, child in ipairs(children) do
		if (child.index == index) then
			child:SetChecked(1)
		else
			child:SetChecked(nil)
		end
	end
	PlaySound("igMainMenuOptionCheckBoxOn")

	if (not frame:GetParent().selected or noupdate) then
		return
	end
	nrf:SetOption(activemenu, frame.option, index, frame.id, frame:GetParent().selected)
	local func = frame:GetParent().func
	if (func) then
		func()
	end
end

function Nurfed_Options_radioGetSelected(frame)
	if (not frame) then
		frame = this:GetParent()
	end
	local children = { frame:GetChildren() }
	for _, child in ipairs(children) do
		if (child:GetChecked()) then
			return child.index
		end
	end
end

-- panes
function Nurfed_Options_paneOnShow()
	local title = getglobal(this:GetName().."title")
	if (this.text) then
		title:SetText(this.text)
	end
	local scroll = getglobal(this:GetName().."scrollScrollBar")
	if (scroll) then
		scroll:SetPoint("RIGHT", scroll:GetParent():GetName(), "RIGHT", -26, 0)
	end
	local children = { this:GetChildren() }
	for _, child in ipairs(children) do
		local objtype = child:GetObjectType()
		if (objtype == "CheckButton") then
			child:SetChecked(child.default)
		elseif (objtype == "Slider") then
			local low = getglobal(child:GetName().."Low")
			local high = getglobal(child:GetName().."High")
			low:SetText(child.low)
			high:SetText(child.high)
			child:SetMinMaxValues(child.min, child.max)
			child:SetValueStep(child.step)
			child:SetValue(child.default)
		elseif (objtype == "Frame" and child.isradio) then
			Nurfed_Options_radioOnClick(child, child.default, true)
		end
		child.option = this.option
	end
	this:SetScript("OnShow", nil)
end

function Nurfed_Options_paneUpdateOptions(frame)
	local option = nrf:getopt(activemenu, frame.option)
	local selected = option[frame.selected]
	local children = { frame:GetChildren() }
	for _, child in ipairs(children) do
		local objtype = child:GetObjectType()
		if (objtype == "CheckButton") then
			child:SetChecked(selected[child.id])
		elseif (objtype == "Slider") then
			child:SetValue(selected[child.id])
		elseif (objtype == "Frame" and child.isradio) then
			Nurfed_Options_radioOnClick(child, selected[child.id], true)
		end
	end
end

function Nurfed_Options_panegetopts()
	local tbl = {}
	local children = { this:GetParent():GetChildren() }
	for _, child in ipairs(children) do
		if (child.id) then
			local objtype = child:GetObjectType()
			if (objtype == "Slider") then
				tbl[child.id] = child:GetValue()
			elseif (objtype == "CheckButton") then
				if (child:GetChecked()) then
					tbl[child.id] = 1
				else
					tbl[child.id] = 0
				end
			elseif (objtype == "Frame" and child.isradio) then
				tbl[child.id] = Nurfed_Options_radioGetSelected(child)
			end
		end
	end
	return tbl
end

function Nurfed_Options_paneAddOption()
	local frame = this:GetParent()
	local objtype = this:GetObjectType()
	if (objtype == "EditBox") then
		if (this:GetText() and this:GetText() ~= "") then
			if (frame.up) then
				this:SetText(string.gsub(this:GetText(), "^%l", string.upper))
			end
			if (frame.notbl) then
				nrf:SetOption(activemenu, frame.option, true, this:GetText())
			else
				local tbl = Nurfed_Options_panegetopts()
				nrf:SetOption(activemenu, frame.option, tbl, this:GetText())
			end
		end
		this:ClearFocus()
		this:SetText("")
	else
		local option = nrf:getopt(activemenu, frame.option)
		local name = table.getn(option) + 1
		local tbl = Nurfed_Options_panegetopts()
		nrf:SetOption(activemenu, frame.option, tbl, name)
	end
	Nurfed_Options_ScrollPane(frame)
	if (frame.func) then
		frame.func()
	end
end

function Nurfed_Options_paneRemoveOption()
	local frame = this:GetParent()
	if (frame.selected) then
		nrf:SetOption(activemenu, frame.option, nil, frame.selected)
		frame.selected = nil
	end
	Nurfed_Options_ScrollPane(frame)
	if (frame.func) then
		frame.func()
	end
end

function Nurfed_Options_ScrollPane(frame)
	if (not frame) then
		frame = this:GetParent()
	else
		this = getglobal(frame:GetName().."scroll")
	end
	local rows = frame.rows
	local selected = frame.selected
	local line, offset, row, text, count, temp
	local option = nrf:getopt(activemenu, frame.option)
	
	if (table.getn(option) > 0) then
		count = table.getn(option)
	else
		temp = {}
		for k in pairs(option) do
			table.insert(temp, k)
		end
		count = table.getn(temp)
	end
	FauxScrollFrame_Update(this, count, rows, 13)
	for line = 1, rows do
		offset = line + FauxScrollFrame_GetOffset(this)
		row = getglobal(frame:GetName().."row"..line)
		text = getglobal(row:GetName().."text")
		if offset <= count then
			if (temp) then
				text:SetText(temp[offset])
				if (selected == temp[offset]) then
					row:LockHighlight()
				else
					row:UnlockHighlight()
				end
			else
				text:SetText(frame.prefix.." "..offset)
				row.id = offset
				if (selected == offset) then
					row:LockHighlight()
				else
					row:UnlockHighlight()
				end
			end
			row:Show()
		else
			row:Hide()
		end
	end
end

function Nurfed_Options_PaneSelect()
	local frame = this:GetParent()
	local selected = frame.selected
	if (this.id) then
		if (selected == this.id) then
			frame.selected = nil
			this:UnlockHighlight()
		else
			frame.selected = this.id
			this:LockHighlight()
		end
	else
		local text = getglobal(this:GetName().."text"):GetText()
		if (selected == text) then
			frame.selected = nil
			this:UnlockHighlight()
		else
			frame.selected = text
			this:LockHighlight()
		end
	end
	if (frame.selected) then
		Nurfed_Options_paneUpdateOptions(frame)
	end
	Nurfed_Options_ScrollPane(frame)
end

-- Dropdowns
function Nurfed_Options_dropdownOnShow()
	UIDropDownMenu_SetWidth(this.width, this)
	UIDropDownMenu_Initialize(this, this.init)
	if (this.selected) then
		UIDropDownMenu_SetSelectedID(this, this.selected)
	end
	UIDropDownMenu_JustifyText("LEFT", this)
end




--[[ Init ]]--
for k, v in pairs(templates) do nrf:createtemp(k, v) end
local frame = nrf:create("Nurfed_OptionsFrame", layout)
table.insert(UISpecialFrames, "Nurfed_OptionsFrame")
frame:RegisterForDrag("LeftButton")
Nurfed_OptionsFrameheadertitle:SetText("Nurfed Options Menu")
Nurfed_OptionsFrameheaderversion:SetText(GetAddOnMetadata("Nurfed_Options", "Version"))
templates = nil
layout = nil

local nrf_cursor = function(a, b)
	local link
	if a and b then
		link = GetContainerItemLink(a, b)
	else
		link = GetInventoryItemLink("player", a)
	end
	if link then
		cursoritem = GetItemInfo(link)
	end
end

hooksecurefunc("PickupContainerItem", nrf_cursor)
hooksecurefunc("PickupInventoryItem", nrf_cursor)
hooksecurefunc("SplitContainerItem", nrf_cursor)
hooksecurefunc("ClearCursor", function() cursoritem = nil end)