--[[-------------------------------------------------------------------------
  Copyright (c) 2006, Jim Whitehead II <cladhaire@gmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of PerfectRaid nor the names of its contributors 
        may be used to endorse or promote products derived from this software
        without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

local frames = {}
local unavail = {}
local aggro = {}

local L = PerfectRaidLocals
local utils = DongleStub("DongleUtils")
PerfectRaid = DongleStub("Dongle"):New("PerfectRaid")

function PerfectRaid:Initialize()
	self.frames = frames
	self.aggro = aggro
	self.defaults = {
		profile = {
			headers = {},
			positions = {},
		},
	}

	self.headerDefaults = {
		Reverse = false,
		ColorClass = false,
		ColorSeverity = true,
	}

	if not ClickCastFrames then
		ClickCastFrames = {}
	end
end

function PerfectRaid:Enable()
	self.db = self:InitializeDB("PerfectRaidDB", self.defaults)

	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MAXHEALTH")
	self:RegisterEvent("UNIT_MANA", "UNIT_MANA")
	self:RegisterEvent("UNIT_RAGE", "UNIT_MANA")
	self:RegisterEvent("UNIT_ENERGY", "UNIT_MANA")
	self:RegisterEvent("UNIT_FOCUS", "UNIT_MANA")
	self:RegisterEvent("UNIT_MAXMANA")
	self:RegisterEvent("UNIT_MAXRAGE", "UNIT_MAXMANA")
	self:RegisterEvent("UNIT_MAXENERGY", "UNIT_MAXMANA")
	self:RegisterEvent("UNIT_MAXFOCUS", "UNIT_MAXMANA")

	--TODO: Make this so we can actually instantiate frames as the users wants
	for i=1,8 do
		local name = "PRHeader"..i
		self.db.profile.headers[name] = setmetatable({}, {__index=self.headerDefaults})
		local header = self:CreateRaidFrame(name, "파티 "..i, tostring(i), nil, PRHeader1)
	end

	PRHeader1:ClearAllPoints()
	self:RestorePosition("PRHeader1")
	PRHeader2:SetPoint("TOP", PRHeader1, "BOTTOM", 0, -20)
	PRHeader3:SetPoint("TOP", PRHeader2, "BOTTOM", 0, -20)
	PRHeader4:SetPoint("TOP", PRHeader3, "BOTTOM", 0, -20)
	PRHeader5:SetPoint("TOPLEFT", PRHeader1, "TOPRIGHT", 10, 0)
	PRHeader6:SetPoint("TOP", PRHeader5, "BOTTOM", 0, -20)
	PRHeader7:SetPoint("TOP", PRHeader6, "BOTTOM", 0, -20)
	PRHeader8:SetPoint("TOP", PRHeader7, "BOTTOM", 0, -20)
	--local header = self:CreateRaidFrame("PRHeader1", "1,2,3,4,5,6,7,8", nil, nil, "CLASS", "WARRIOR,PRIEST,PALADIN,DRUID,SHAMAN,ROGUE,MAGE,WARLOCK,HUNTER", 2, 10, "LEFT")
	--self:RestorePosition("PRHeader1")
end

function PerfectRaid:SavePosition(name)
    local f = getglobal(name)
    local x,y = f:GetLeft(), f:GetTop()
    local s = f:GetEffectiveScale()
    
    x,y = x*s,y*s
    
	local opt = self.db.profile.positions[name]
	if not opt then 
		self.db.profile.positions[name] = {}
		opt = self.db.profile.positions[name]
	end
    opt.PosX = x
    opt.PosY = y
end

function PerfectRaid:RestorePosition(name)
	local f = getglobal(name)
	local opt = self.db.profile.positions[name]
	if not opt then 
		self.db.profile.positions[name] = {}
		opt = self.db.profile.positions[name]
	end

	local x = opt.PosX
	local y = opt.PosY

    local s = f:GetEffectiveScale()
        
    if not x or not y then
        f:ClearAllPoints()
        f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
        return 
    end

    x,y = x/s,y/s

    f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

local function OnDragStart(frame)
	local parent = frame.dragparent
	if not parent.locked then
		parent:StartMoving()
		PerfectRaid.moving = parent
	end
end

local function OnDragStop(frame)
	local parent = frame.dragparent
	parent:StopMovingOrSizing()
	PerfectRaid:SavePosition(parent:GetName())
	PerfectRaid.moving = nil
end

function PerfectRaid:CreateRaidFrame(name, title, filter, strict, dragparent, group, order, columns, coloffset, colanchor)
	local frame = CreateFrame("Frame", name, UIParent, "SecureRaidGroupHeaderTemplate")
	frame.title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	frame.title:SetPoint("BOTTOM", frame, "TOP", 0, 3)
	frame.title:SetText(title or "")
	frame:SetAttribute("point", "TOP")
	frame:SetAttribute("groupFilter", filter)
	frame:SetAttribute("template", "SecureUnitButtonTemplate")
	frame:SetAttribute("templateType", "Button")
	frame:SetAttribute("yOffset", -2)
	frame:SetAttribute("sortMethod", "NAME")
	frame:SetAttribute("strictFiltering", strict)
	frame:SetAttribute("groupBy", group)
	frame:SetAttribute("groupingOrder", order)
	frame:SetAttribute("maxColumns", columns)
	frame:SetAttribute("columnSpacing", coloffset)
	frame:SetAttribute("columnAnchorPoint", colanchor)
	frame.dragparent = dragparent or frame
	frame.initialConfigFunction = PerfectRaid.ConfigureButton
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:Show()
	return frame
end

function PerfectRaid:GetColoredName(unit)
	local class = select(2, UnitClass(unit)) or "WARRIOR"
	local color = RAID_CLASS_COLORS[class]
	return string.format("|cFF%02x%02x%02x%s|r", color.r*255, color.g*255, color.b*255, UnitName(unit) or "Unknown")
end

function PerfectRaid:UNIT_HEALTH(event, unit)
	if not frames[unit] then return end
	local health = UnitHealth(unit)
	local max = UnitHealthMax(unit)

	-- Hack to fix api issue
	if max < health then max = health end
	local deficit = max - health
	local perc = UnitHealthMax(unit) / health
	local class = select(2, UnitClass(unit))
	local perc = (health/max) or 0

	if deficit > 999 then 
		deficit = string.format("%.1fk", deficit / 1000)
	elseif deficit == 0 then
		deficit = ""
	end

	--TODO: Seperate this out properly into the logical module
	if aggro[unit] and aggro[unit] >= 15 then
		if deficit == "" then
			deficit = L["Aggro"]
		else
			deficit = "|cFFFF1111"..deficit.."|r"
		end
	end

	if UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then
		local status

		unavail[unit] = true

		if UnitIsDead(unit) then 
			status = L["Dead"]
		elseif UnitIsGhost(unit) then
			status = L["Ghost"]
		elseif not UnitIsConnected(unit) then
			status = L["Offline"]
		end			

		for frame in pairs(frames[unit]) do
			frame.healthbar:SetValue(max)
			frame.manabar:SetValue(max)
			frame.status:SetText(status)
			frame.healthbar:SetStatusBarColor(0.3, 0.3, 0.3)
			frame.manabar:SetStatusBarColor(0.3, 0.3, 0.3)
			frame:SetBackdropBorderColor(0.3, 0.3, 0.3)
		end
	else
		if unavail[unit] then
			for frame in pairs(frames[unit]) do
				local options = self.db.profile.headers[frame.headerName]
				if options.ColorClass then
					local c = frame.classcolor
					frame.healthbar:SetStatusBarColor(c.r,c.g,c.b)
				elseif options.ColorSeverity then
					frame.healthbar:SetStatusBarColor(utils.GetHPSeverity(perc))
				end

				local value = health
				if options.Reverse then
					value = max - health
				end

				frame.healthbar:SetMinMaxValues(0, max)
				frame.healthbar:SetValue(value)
				frame.status:SetText(nil)
			end
			unavail[unit] = nil
		end

		for frame in pairs(frames[unit]) do
			local options = self.db.profile.headers[frame.headerName]
			if options.ColorSeverity then
				frame.healthbar:SetStatusBarColor(utils.GetHPSeverity(perc))
			end

			local value = health
			if options.Reverse then
				value = max - health
			end

			frame.healthbar:SetValue(value)
			frame.status:SetText(deficit)
		end
	end
end

function PerfectRaid:UNIT_MANA(event, unit)
	if not frames[unit] then return end
	local mana = UnitMana(unit)
	
	for frame in pairs(frames[unit]) do
		frame.manabar:SetValue(mana)
	end
end

function PerfectRaid:UNIT_MAXHEALTH(event, unit)
	if not frames[unit] then return end
	local max = UnitHealthMax(unit)
	local health = UnitHealth(unit)
	for frame in pairs(frames[unit]) do
		frame.healthbar:SetMinMaxValues(0, max)
		frame.healthbar:SetValue(health)
	end
end

function PerfectRaid:UNIT_MAXMANA(event, unit)
	if not frames[unit] then return end
	local max = UnitManaMax(unit)
	local mana = UnitMana(unit)
	for frame in pairs(frames[unit]) do
		frame.manabar:SetMinMaxValues(0, max)
		frame.manabar:SetValue(mana)
	end
end

local function OnShow(frame)
	local self = PerfectRaid
	local unit = frame:GetAttribute("unit")
	frame.name:SetText(self:GetColoredName(unit))
--	frame.name:SetText(UnitName(unit))

	local idx = tonumber(string.match(unit, "^raid(%d+)$"))
	if idx then
		local group = select(3, GetRaidRosterInfo(idx))
		frame.group:SetText(group)
	end

	local class = select(2, UnitClass(unit)) or "WARRIOR"
	frame.class = class 
	frame.classcolor = RAID_CLASS_COLORS[class]
	frame.manacolor = ManaBarColor[UnitPowerType(unit)] 

	local color = frame.classcolor
	frame.healthbar:SetStatusBarColor(color.r, color.g, color.b)	
	frame.healthbar:SetMinMaxValues(0, UnitHealthMax(unit))
	frame.healthbar:SetValue(UnitHealth(unit))
	frame:SetBackdropBorderColor(color.r, color.g, color.b)

    local color = frame.manacolor
    frame.manabar:SetStatusBarColor(color.r, color.g, color.b)
	frame.manabar:SetMinMaxValues(0, UnitManaMax(unit))
	frame.manabar:SetValue(UnitMana(unit))

	frame.status:SetText(nil)

	frames[unit] = frames[unit] or {}
	frames[unit][frame] = true

	for name,module in self:IterateModules() do
		if type(module.ShowButton) == "function" then
			module:ShowButton(frame)
		end
	end

	self:UNIT_HEALTH(nil, unit)
	self:UNIT_MANA(nil, unit)
end

local function OnHide(frame)
	local unit = frame:GetAttribute("unit")
	frames[unit][frame] = nil
end

local function OnAttributeChanged(frame, name, value)
	if name ~= "unit" then return end

	for unit,tbl in pairs(frames) do
		if tbl[frame] and frame:GetAttribute("unit") ~= unit then
			tbl[frame] = nil
		end
	end

	local parent = frame:GetParent()
	if value == nil then
		local num = parent:GetNumChildren()
		local visible = nil
		for i=1,num do
			local button = parent:GetAttribute("child"..i)
			if button and button:IsShown() then visible = true end
		end
		if parent.title then
			if not visible then
				parent.title:Hide()
			else
				parent.title:Show()
			end
		end
	else
		parent.title:Show()
	end
end

function PerfectRaid.ConfigureButton(button)
	local self = PerfectRaid

	button:SetWidth(200)
	button:SetHeight(14)
--	button:SetBackdrop(GameTooltip:GetBackdrop())
	button:SetBackdropColor(0.3, 0.3, 0.3)
	button:SetBackdropBorderColor(0.4, 0.4, 0.4)

	ClickCastFrames[button] = true
	button:SetAttribute("*type1", "target")

	local parent = button:GetParent()
	button.dragparent = parent.dragparent
	button.header = parent
	button.headerName = parent:GetName()

	local leftbox = CreateFrame("Frame", nil, button)
	leftbox:SetPoint("TOPLEFT", 0, 0)
	leftbox:SetPoint("BOTTOMRIGHT", button, "BOTTOMLEFT", 70, 0)
	button.leftbox = leftbox

	local rightbox = CreateFrame("Frame", nil, button)
	rightbox:SetPoint("TOPRIGHT", 0, 0)
	rightbox:SetPoint("BOTTOMLEFT", button, "BOTTOMRIGHT", -40, 0)
	button.rightbox = rightbox

	local bar = CreateFrame("StatusBar", nil, button)
	bar:SetPoint("TOPLEFT", leftbox, "TOPRIGHT", 0, 0)
	bar:SetPoint("BOTTOMRIGHT", rightbox, "BOTTOMLEFT", 0, 0)
	bar:SetStatusBarTexture("Interface\\AddOns\\PerfectRaid\\images\\smooth")
	button.healthbar = bar

	local bar = CreateFrame("StatusBar", nil, button.healthbar)
	bar:SetPoint("BOTTOMLEFT", 0, 0)
	bar:SetPoint("BOTTOMRIGHT", 0, 0)
	bar:SetHeight(2)
	bar:SetStatusBarTexture("Interface\\AddOns\\PerfectRaid\\images\\smooth")
	button.manabar = bar
--	bar:Hide()

	local group = leftbox:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	group:SetPoint("LEFT", 0, 0)
	group:SetHeight(14)
	group:SetJustifyH("LEFT")
	button.group = group
	button.group:SetWidth(0.1)
	button.group:Hide()

	local font = button.healthbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	font:SetPoint("RIGHT", leftbox, "RIGHT", -2, 0)
	font:SetPoint("LEFT", group, "RIGHT", 0, 0)
	font:SetJustifyH("RIGHT")
	button.name = font

	local raidicon = button:CreateTexture(nil, "ARTWORK")
	raidicon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	raidicon:SetHeight(16)
	raidicon:SetWidth(0.1)
	raidicon:SetPoint("RIGHT", leftbox, "LEFT", 0, 0)
	button.raidicon = raidicon
	raidicon:Hide()

	local leadericon = button:CreateTexture(nil, "ARTWORK")
	leadericon:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	leadericon:SetHeight(16)
	leadericon:SetWidth(0.1)
	leadericon:SetPoint("RIGHT", raidicon, "LEFT", 0, 0)
	button.leadericon = leadericon
	leadericon:Hide()

	local looticon = button:CreateTexture(nil, "ARTWORK")
	looticon:SetTexture("Interface\\GroupFrame\\UI-Group-MasterLooter")
	looticon:SetHeight(16)
	looticon:SetWidth(0.1)
	looticon:SetPoint("RIGHT", leadericon, "LEFT", 0, 0)
	button.looticon = looticon
	looticon:Hide()

	local font = button.healthbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	font:SetPoint("RIGHT", -2, 2)
	button.status = font

	button:SetClampedToScreen(true)
	button:RegisterForDrag("LeftButton")

	for name,module in self:IterateModules() do
		if type(module.ConfigureButton) == "function" then
			module:ConfigureButton(button)
		end
	end

	button:SetScript("OnShow", OnShow)
	button:SetScript("OnDragStart", OnDragStart)
	button:SetScript("OnDragStop", OnDragStop)
	button:SetScript("OnAttributeChanged", OnAttributeChanged)
end

function PerfectRaid:RAID_ROSTER_UPDATE()
	if self.moving then
		self.moving:StopMovingOrSizing()
	end
end
