﻿--[[-------------------------------------------------------------------------
  Copyright (c) 2006, Jim Whitehead II <cladhaire@gmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of PerfectRaid nor the names of its contributors 
        may be used to endorse or promote products derived from this software
        without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

local Hacks = PerfectRaid:NewModule("PerfectRaid-Hacks")
local frames

function Hacks:Initialize()
	PerfectRaid.defaults.profile.HacksScale = 0.75
	PerfectRaid.defaults.profile.HacksBackdrop = false

	frames = PerfectRaid.frames
end

function Hacks:ShowButton(button)
	self:UpdateScale()
	self:UpdateBackdrop()
end

local options
function Hacks:CreateOptions(opt)
	options = CreateFrame("Frame", "PROptions_Hacks", PROptions)
	options:SetScript("OnShow", function() self:OnShow() end)

	opt:AddOptionsTab("크기", options)

	options.widgets = {}

	local slider = CreateFrame("Slider", "PRHacks_Scale", options, "PRSliderTemplate")
	slider.Text:SetText("프레임 크기")
	slider.High:SetText("2.0")
	slider.Low:SetText("0.1")
	slider:SetMinMaxValues(0.1,2.0)
	slider:SetValueStep(0.05)
	table.insert(options.widgets, slider)

	local check = CreateFrame("CheckButton", "PRHacks_Backdrop", options, "PRCheckTemplate")
	check.Label:SetText("레이드배경 표시")
	table.insert(options.widgets, check)

	slider:SetScript("OnValueChanged", function(frame)
		frame.Current:SetText(string.format("%0.2f", frame:GetValue()))
		self:UpdateScale(frame:GetValue())
	end)

	local cancel = CreateFrame("Button", "PRHacks_Cancel", options, "PRButtonTemplate")
	cancel.Label:SetText("취소")
	cancel:SetPoint("BOTTOMRIGHT", 0, 5)
	cancel:SetScript("OnClick", function() self:OnShow() end)
	cancel:Show()
	
	local save = CreateFrame("Button", "PRHacks_Save", options, "PRButtonTemplate")
	save.Label:SetText("저장")
	save:SetPoint("BOTTOMRIGHT", cancel, "BOTTOMLEFT", -10, 0)
	save:SetScript("OnClick", function() self:SaveOptions() end)
	save:Show()

	for idx,widget in ipairs(options.widgets) do
		widget:Show()
		if idx == 1 then
			widget:SetPoint("TOPLEFT", 0, -5)
		else
			widget:SetPoint("TOPLEFT", options.widgets[idx - 1], "BOTTOMLEFT", 0, -15)
		end
	end
end

function Hacks:OnShow()
	local profile = PerfectRaid.db.profile

	PRHacks_Scale:SetValue(profile.HacksScale)
	PRHacks_Backdrop:SetChecked(profile.HacksBackdrop)
	self:UpdateScale()
	self:UpdateBackdrop()
end

function Hacks:SaveOptions()
	local profile = PerfectRaid.db.profile
	
	profile.HacksScale = PRHacks_Scale:GetValue()
	profile.HacksBackdrop = PRHacks_Backdrop:GetChecked()
	self:UpdateScale()
	self:SavePositions()
	self:UpdateBackdrop()
end

function Hacks:UpdateBackdrop()
	if not PRHeader1 or not PRHeader8 or not PRHeader4 or not PRHeader8:GetRect() then return end
	if not self.backdrop then
		self.backdrop = PRHeader1:CreateTexture(nil, "BACKGROUND")
		self.backdrop:SetPoint("TOPLEFT", PRHeader1, "TOPLEFT", -5 , 15)
		self.backdrop:SetPoint("RIGHT", PRHeader8, "RIGHT", 3, 0)
		if PRHeader4:GetBottom() < PRHeader8:GetBottom() then
			self.backdrop:SetPoint("BOTTOM", PRHeader4, "BOTTOM", 0, -10)
		else
			self.backdrop:SetPoint("BOTTOM", PRHeader8, "BOTTOM", 0, -10)
		end
		self.backdrop:SetTexture(0,0,0,1)
	end
	if PerfectRaid.db.profile.HacksBackdrop then
		self.backdrop:Show()
	else
		self.backdrop:Hide()
	end
end

function Hacks:UpdateScale(val)
	local scale = val or PerfectRaid.db.profile.HacksScale
	for i=1,8 do
		local name = "PRHeader"..i
		local button = getglobal(name)
		if button then
			button:SetScale(scale)
		end
	end
end

function Hacks:SavePositions()
	for i=1,8 do
		local name = "PRHeader"..i
		local button = getglobal(name)
		if button then
			PerfectRaid:SavePosition(name)
		end
	end
end

