--[[-------------------------------------------------------------------------
  Copyright (c) 2006, Jim Whitehead II <cladhaire@gmail.com>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of PerfectRaid nor the names of its contributors 
        may be used to endorse or promote products derived from this software
        without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

local Options = PerfectRaid:NewModule("PerfectRaid-Options")

function Options:Initialize()
end

function Options:Enable()
	self.cmd = self:InitializeSlashCommand("PerfectRaid Options", "PERFECTRAID", "praid", "perfectraid")
	self.cmd:RegisterSlashHandler("Show options GUI", ".*", "ShowOptions") 
end

function Options:ShowOptions()
	if not PROptions then
		self:CreateOptions()
		PROptions:Show()
	elseif PROptions:IsVisible() then
		PROptions:Hide()
	else
		PROptions:Show()
	end
end

function Options:CreateOptions()
	local frame = CreateFrame("Frame", "PROptions", UIParent, "PROptionsTemplate")
	frame:SetHeight(300)
	frame:SetWidth(450)
	frame:SetPoint("CENTER", 0, 0)

	for name,module in PerfectRaid:IterateModules() do
		if module ~= self and type(module.CreateOptions) == "function" then
			module:CreateOptions(self)
		end
	end

	local button = CreateFrame("Button", "PROptionsClose", PROptions, "UIPanelCloseButton")
	button:SetHeight(25)
	button:SetWidth(25)
	button:SetPoint("TOPRIGHT", -5, 3)
	button:SetScript("OnClick", function() PROptions:Hide() end)
end

local tabs = {}
function Options:AddOptionsTab(title, frame)
	local num = #tabs + 1
	local name = "PROptionsTab"..num
	local tab = CreateFrame("Button", name, PROptions, "PRTabTemplate")
	
	if num == 1 then
		tab:SetPoint("TOPLEFT", PROptions, "BOTTOMLEFT", 10, 12)
	else
		local prev = getglobal("PROptionsTab"..num-1)
		tab:SetPoint("TOPLEFT", prev, "TOPRIGHT", 3, 0)
	end
	
	tab.Name:SetText(title)
	tab.frame = frame
	local width = tab.Name:GetWidth()
	tab:SetWidth(width + 30)
	tab:Show()

	table.insert(tabs, tab)
end

local tabselected
function Options:TabOnClick(tab)
	if tabselected then
		tabselected.Left:SetTexture("Interface\\AddOns\\PerfectRaid\\images\\OrangeTheme\\TabLeftInactive")
		tabselected.Middle:SetTexture("Interface\\AddOns\\PerfectRaid\\images\\OrangeTheme\\TabMiddleInactive")
		tabselected.Right:SetTexture("Interface\\AddOns\\Perfectraid\\images\\OrangeTheme\\TabRightInactive")
		tabselected.frame:Hide()
	end
	tab.Left:SetTexture("Interface\\AddOns\\PerfectRaid\\images\\OrangeTheme\\TabLeft")
	tab.Middle:SetTexture("Interface\\AddOns\\PerfectRaid\\images\\OrangeTheme\\TabMiddle")
	tab.Right:SetTexture("Interface\\AddOns\\Perfectraid\\images\\OrangeTheme\\TabRight")
	tab.frame:SetPoint("TOPLEFT", 15, -25)
	tab.frame:SetPoint("BOTTOMRIGHT", -15, 15)
	tab.frame:Show()
	tabselected = tab
end
